---------------------------------------------------------------------------
--
-- Licensed Materials - Property of IBM
-- 5724-T05
--
-- (C) Copyright IBM Corp. 2003,2010. All Rights Reserved.
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
---------------------------------------------------------------------------

--
-- table of PM_User_Group
-- added for Enterprise version
--

CREATE TABLE pm_group
 (
   group_id           INTEGER          NOT NULL,
   group_name         VARCHAR2(256),
    --
    -- primary key
    --
   CONSTRAINT PK_pm_group PRIMARY KEY(group_id)
 )
TABLESPACE WM_Client;

CREATE UNIQUE INDEX UNQ_pm_grp_name on pm_group(UPPER(group_name))
TABLESPACE WM_Client;

--
-- create default groups
--
INSERT INTO pm_group(group_id,group_name)
VALUES(1,'Admin');

INSERT INTO pm_group(group_id,group_name)
VALUES(2,'Everybody');

CREATE SEQUENCE seq_pm_group_id
    START WITH  1000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE
;

--
-- table of users
--
CREATE TABLE PM_User(
    Id               INTEGER       NOT NULL,
    FolderId         INTEGER,
    Name             VARCHAR2(64)  NOT NULL,
    Description      VARCHAR2(255),
    Password         VARCHAR2(64),
    --
    -- Type:
    --   'N' - Normal User
    --   'D' - Data Administrator
    --   'S' - System Administrator
    --   'E' - SES User
    --   'L' - Local Enterprise
    --   'R' - Remote Enterprise
    --
    Type             CHAR(1)       DEFAULT 'N' NOT NULL,
      CONSTRAINT PM_User_Type_ck CHECK (Type IN ('N','D','S', 'E','L','R')),
    --
    -- AutoPurgeRequired:
    --   'Y' - Purge 'Never' is disallowed
    --   'N' - Purge 'Never' is allowed
    --
    AutoPurgeRequired CHAR(1)      DEFAULT 'N' NOT NULL,
      CONSTRAINT PM_User_AutoPurgeRequired_ck CHECK (AutoPurgeRequired IN ('Y','N')),
    --
    -- MaxPurgeInterval:
    --   =0 - Requires purge on download
    --   >0 - Specifies max days user may retain output after download
    --
    MaxPurgeInterval INTEGER       DEFAULT 999 NOT NULL,
      CONSTRAINT PM_User_MaxPurgeInterval_ck CHECK (MaxPurgeInterval Between 0 And 999),
    --
    -- MaxUsageMB/CurUsageMB specify the max/current server disk usage in MBbytes
    --
    MaxUsageMB       INTEGER       DEFAULT 10 NOT NULL,
      CONSTRAINT PM_User_MaxUsageMB_ck CHECK (MaxUsageMB Between 0 And 999),
    group_id         INTEGER       NOT NULL,
    --
    -- primary key
    --
    CONSTRAINT PK_PM_User PRIMARY KEY(Id),
    -- foreign key to PM_GROUP table
    CONSTRAINT fk_pm_user_grp_id FOREIGN KEY(group_id) REFERENCES pm_group(group_id)
)
TABLESPACE WM_Client;
--
-- index required to prevent duplicate users
--

CREATE UNIQUE INDEX UNQ_User ON PM_User(UPPER(Name))
TABLESPACE WM_Client;

-- SEAde44516: Eliminate deadlocking.
CREATE INDEX pm_user_group_id_idx ON pm_user( group_id );

--
-- sequence for PM_User.Id
--
CREATE SEQUENCE SQ_User
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE
;
--
-- password constraint
--
CREATE OR REPLACE TRIGGER PM_User_TR_BIU
    BEFORE INSERT OR UPDATE ON PM_User
    FOR EACH ROW
    BEGIN
        IF :NEW.Name = 'system' AND :NEW.Password IS NOT NULL THEN
            RAISE_APPLICATION_ERROR(-20001, 'system user cannot have a password');
        ELSIF :NEW.Name != 'system' AND :NEW.Password IS NULL THEN
            RAISE_APPLICATION_ERROR(-20001, 'Password is required');
        END IF;
    END;
    
/
SHOW ERRORS
--
-- table of folders
--
CREATE TABLE PM_Folder(
    Id               INTEGER       NOT NULL,
    FolderId         INTEGER,      -- parent folder id, NULL for ROOT folder
    UserId           INTEGER       NOT NULL,
    Name             VARCHAR2(64)  NOT NULL,
    Description      VARCHAR2(255),
    Changed          DATE          DEFAULT TRUNC(SYSDATE,'MI') NOT NULL,
    --
    -- Type:
    --   'N' - Normal Folder
    --   'S' - Shared Folder
    --
    Type             CHAR(1)       DEFAULT 'N' NOT NULL,
      CONSTRAINT PM_Folder_Type_ck CHECK (Type IN ('N','S')),
    Category         VARCHAR2(255),
    Retain           CHAR(1)       DEFAULT 'Y' NOT NULL,
    RetainDays       INTEGER       NULL,
    InheritRetention CHAR(1)       DEFAULT 'Y' NOT NULL,
    --
    -- constraints required to prevent removing folder or user
    --
    CONSTRAINT FK_Folder_FolderId
    FOREIGN KEY(FolderId)          REFERENCES PM_Folder(Id),
    CONSTRAINT FK_Folder_UserId
       FOREIGN KEY(UserId)            REFERENCES PM_User(Id),
    CONSTRAINT CK_Folder_Retain       
       CHECK  (Retain IN ('Y', 'N')),
    CONSTRAINT CK_Folder_Inherit_Retention
       CHECK (InheritRetention IN ('Y', 'N')),
    --
    -- primary key
    --
    CONSTRAINT PK_PM_Folder PRIMARY KEY(Id)
)
TABLESPACE WM_Client;
--
-- constraint required to prevent removing a user's home folder
--
ALTER TABLE PM_User ADD CONSTRAINT C1_User
    FOREIGN KEY(FolderId) REFERENCES PM_Folder(Id)
;
--
-- index required to prevent duplicate sub-folders in folder
--
CREATE UNIQUE INDEX UNQ_Folder ON PM_Folder(FolderId, UPPER(Name))
TABLESPACE WM_Client;
--
-- Indexes to prevent deadlocking in pm_group_user.deleteuser
CREATE INDEX pm_user_folderid_idx ON PM_USER(FOLDERID);

CREATE INDEX pm_folder_userid_idx ON PM_FOLDER(USERID);

-- sequence for PM_Folder.Id
--
CREATE SEQUENCE SQ_Folder
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE
;
--
-- create view of PM_Folder with outer self join to get count of sub-folders
--
CREATE VIEW VW_Folder AS
SELECT
    f.Id,
    f.FolderId,
    f.UserId,
    f.Name,
    f.Description,
    f.Changed,
    f.Category,
    f.Type,
    COUNT(c.Id) Children
FROM
    PM_Folder f,
    PM_Folder c 
WHERE 
    f.Id = c.FolderId(+) AND
    c.id(+) > 0 AND            -- SEAde18976
    f.Id > 0                   -- 
GROUP BY
    f.Id,
    f.FolderId,
    f.UserId,
    f.Name,
    f.Description,
    f.Changed,
    f.Category,
    f.Type;

CREATE TABLE PM_FileFormat(
    Id               INTEGER       NOT NULL,
    --
    -- FileFormat:
    --   if ExcelFormat='Y' - Excel FileFormat
    --   if ExcelFormat='N' - Custom FileFormat (e.g., Raw)
    --
    FileFormat       INTEGER       NOT NULL,
    ExcelFormat      CHAR(1)       NOT NULL,
      CONSTRAINT PM_FileFormat_ExcelFormat_ck CHECK (ExcelFormat IN ('Y','N')),
    UsesDelim        CHAR(1)       NOT NULL,
      CONSTRAINT PM_FileFormat_UsesDelim_ck CHECK (UsesDelim IN ('Y','N')),
    Name             VARCHAR2(64)  NOT NULL,
    --
    -- primary key
    --
    CONSTRAINT PK_PM_FileFormat PRIMARY KEY(Id)
)
TABLESPACE WM_Client;

-- SEAde20706 support emailing of reports

CREATE TABLE PM_Email(
    EmailId             INTEGER CONSTRAINT PK_PM_Email PRIMARY KEY,
    Subject             VARCHAR2(255) ,
    Message             VARCHAR2(4000)
    )
TABLESPACE WM_Client;

CREATE SEQUENCE SQ_Email
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE;

CREATE TABLE pm_email_address(
  AddressId   INTEGER ,
  EmailId     INTEGER ,
  address     VARCHAR2(128),
    CONSTRAINT FK_address_EmailID
    FOREIGN KEY(EmailId) REFERENCES PM_Email(EmailId) ON DELETE CASCADE,
    CONSTRAINT PK_PM_EmailAddr PRIMARY KEY(AddressId)
) TABLESPACE WM_Client;

CREATE SEQUENCE SQ_EmailAddress
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE;

--

CREATE TABLE PM_Notification(
    Id               INTEGER,      
    FolderId         INTEGER,    
    UserId           INTEGER, 
    Name             VARCHAR2(64),
    Description      VARCHAR2(255),
    Changed          DATE          DEFAULT TRUNC(SYSDATE,'MI') NOT NULL,
    Type             VARCHAR(64),
    Category         VARCHAR2(64),
    CONSTRAINT FK_notification_FolderId
                FOREIGN KEY(FolderId)          REFERENCES PM_Folder(Id),
    CONSTRAINT FK_notification_UserId
                FOREIGN KEY(UserId)            REFERENCES PM_User(Id),
    --
    -- primary key
    --
    CONSTRAINT PK_PM_notification PRIMARY KEY(Id)   
) TABLESPACE WM_Client;

CREATE SEQUENCE SQ_notification
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE;

--
-- index required to prevent duplicate notifications in folder
--
CREATE UNIQUE INDEX UNQ_Notification ON PM_Notification(FolderId, UPPER(Name))
TABLESPACE WM_Client;

CREATE TABLE PM_Email_Notification(
        email_notify_id  INTEGER,
        Email_Id         INTEGER NOT NULL,
  Notify_Id        INTEGER NOT NULL,
    CONSTRAINT pk_email_notify PRIMARY KEY (email_notify_id),
    CONSTRAINT fk_email_notify_email_id FOREIGN KEY(email_id) 
     REFERENCES pm_email(emailid) ON DELETE CASCADE ,
    CONSTRAINT fk_email_notify_id FOREIGN KEY(Notify_Id) 
     REFERENCES PM_Notification(Id) 
) TABLESPACE WM_Client;

CREATE SEQUENCE SQ_Email_Notification
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE;

CREATE TABLE PM_Destination(
    Id               INTEGER       NOT NULL,
    FormatId         INTEGER       NOT NULL,
    ToDisplay        CHAR(1)       NOT NULL,
      CONSTRAINT PM_Destination_ToDisplay_ck CHECK (ToDisplay IN ('Y','N')),
    ToPrinter        CHAR(1)       NOT NULL,
      CONSTRAINT PM_Destination_ToPrinter_ck CHECK (ToPrinter IN ('Y','N')),
    ToFile           CHAR(1)       NOT NULL,
      CONSTRAINT PM_Destination_ToFile_ck CHECK (ToFile IN ('Y','N')),
    ToEmail          CHAR(1)       DEFAULT 'N' NOT NULL,
      CONSTRAINT PM_Destination_ToEmail_ck CHECK (ToEmail IN ('Y','N')),
    Delimiter        CHAR(1)       DEFAULT ' ' NOT NULL,
    Directory        VARCHAR2(255),
    EmailId          INTEGER,
    --
    -- foreign key(s)
    --
    CONSTRAINT FK_Destination_FormatId
    FOREIGN KEY(FormatId)          REFERENCES PM_FileFormat(Id),
    --CONSTRAINT FK_Destination_EmailId
    --FOREIGN KEY(EmailId) REFERENCES PM_Email(EmailId) ON DELETE CASCADE,
    -- primary key
    --
    CONSTRAINT PK_PM_Destination PRIMARY KEY(Id)
)
TABLESPACE WM_Client;
--
-- sequence for PM_Destination.Id
--
CREATE SEQUENCE SQ_Destination
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE
;

CREATE TABLE PM_PageSetup(
    Id               INTEGER       NOT NULL,
    --
    -- Orientation:
    --   'P' - Portrait
    --   'L' - Landscape
    --
    Orientation      CHAR(1)       DEFAULT 'P' NOT NULL,
      CONSTRAINT PM_PageSetup__ck CHECK (Orientation IN ('P','L')),
    --
    -- ScalingType:
    --   'F' - Fit-to-page(s)
    --   'M' - Manual
    --
    ScalingType      CHAR(1)       DEFAULT 'M' NOT NULL,
      CONSTRAINT PM_PageSetup_ScalingType_ck  CHECK (ScalingType IN ('F','M')),
    ReductionRatio   SMALLINT      DEFAULT 100 NOT NULL,
      CONSTRAINT PM_PageSetup_ReductionRatio_ck  CHECK (ReductionRatio Between 10 And 400),
    PagesWide        SMALLINT      DEFAULT 1 NOT NULL,
      CONSTRAINT PM_PageSetup_PagesWide_ck  CHECK (PagesWide Between 1 And 99),
    PagesTall        SMALLINT      DEFAULT 1 NOT NULL,
      CONSTRAINT PM_PageSetup_PagesTall_ck CHECK (PagesTall Between 1 And 9999),
    StyleSheet       VARCHAR2(255),
    UserMacro        VARCHAR2(255),
    SplitKey         VARCHAR2(256), -- for breaking traffic keys into sub components
    HideKey          CHAR(1) DEFAULT 'N' NOT NULL, -- hide the composite key in first column on traffic reports
      CONSTRAINT PM_PageSetup_HideKey_ck  CHECK (HideKey IN ('Y','N')),
    --
    -- primary key
    --
    CONSTRAINT PK_PM_PageSetup PRIMARY KEY(Id)
)
TABLESPACE WM_Client;
--
-- sequence for PM_PageSetup.Id
--
CREATE SEQUENCE SQ_PageSetup
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE
;

CREATE TABLE PM_ClientMachine(
    Id               INTEGER       NOT NULL,
    Domain           VARCHAR2(255) NOT NULL,
    MachineName      VARCHAR2(255) NOT NULL ,
    --
    -- primary key
    --
    CONSTRAINT PK_PM_ClientMachine PRIMARY KEY(Id)
)
TABLESPACE WM_Client;
--
-- index required
--
CREATE UNIQUE INDEX UNQ_ClientMachine ON PM_ClientMachine(Domain, UPPER(MachineName))
TABLESPACE WM_Client;
--
-- sequence for PM_ClientMachine.Id
--
CREATE SEQUENCE SQ_ClientMachine
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE
;

CREATE TABLE PM_DTScope(
    Id               INTEGER       NOT NULL,
    --
    -- ScopeType:
    --   'C' - Calendar 
    --   'R' - Relative
    --
        ScopeType        CHAR(1)       DEFAULT 'C' NOT NULL,
      CONSTRAINT PM_DTScope_ScopeType_ck  CHECK (ScopeType IN ('C','R')),
    --
    -- StartOfWeek:
    --   Values are between 0 (Sunday) and 6 (Saturday)
    --
        StartOfWeek      INTEGER       DEFAULT NULL,
      CONSTRAINT CHK_DTScope_StartOfWeek_ck CHECK (StartOfWeek BETWEEN 0 AND 6),
    --
    -- TimeUnit:
    --   'H' - Hours 
    --   'D' - Days 
    --   'W' - Weeks 
    --   'M' - Months
    --   One = 'O'
    --   Two = 'B'
    --   Five = 'F'
    --   Half = 'P'
    --   Quarter = 'Q'
    --   Ten = 'T'
    --   Three = 'R'
    --   Six = 'I'
    --   Twenty  = 'E'
    --   
    TimeUnit         CHAR(1)       DEFAULT 'H' NOT NULL,
      CONSTRAINT PM_DTScope_TimeUnit_ck CHECK (TimeUnit IN ('H','D','W','M','O','B','F','P','Q','T','R','I','E')),               
    HowManyUnits     INTEGER       DEFAULT 1 NOT NULL, 
    IncludeCurrent   CHAR(1)       DEFAULT 'Y' NOT NULL,
      CONSTRAINT PM_DTScope_IncludeCurrent_ck CHECK (IncludeCurrent IN ('Y','N')),
    SameHoursEachDay CHAR(1)       DEFAULT 'Y' NOT NULL,
      CONSTRAINT PM_DTScope_SameHoursEachDay_ck  CHECK (SameHoursEachDay IN ('Y','N')),
    IncludeHolidays  CHAR(1)       DEFAULT 'Y' NOT NULL,
      CONSTRAINT PM_DTScope_IncludeHolidays_ck  CHECK (IncludeHolidays IN ('Y','N')),
    HolidaysOnly     CHAR(1)       DEFAULT 'N' NOT NULL,
      CONSTRAINT PM_DTScope_HolidaysOnly_ck  CHECK (HolidaysOnly IN ('Y','N')),
    IncludeWeekends  CHAR(1)       DEFAULT 'Y' NOT NULL,
      CONSTRAINT PM_DTScope_IncludeWeekends_ck  CHECK (IncludeWeekends IN ('Y','N')),
    WeekendsOnly     CHAR(1)       DEFAULT 'N' NOT NULL,
      CONSTRAINT PM_DTScope_WeekendsOnly_ck  CHECK (WeekendsOnly IN ('Y','N')),
    DTPeriod         INTEGER       DEFAULT 24 NOT NULL,
    ExtendRange      CHAR(1)       DEFAULT 'N' NOT NULL,
      CONSTRAINT PM_DTScope_ExtendRange_ck  CHECK (ExtendRange IN ('Y','N')),
    QuickRange       CHAR(1)       DEFAULT 'N' NOT NULL,
      CONSTRAINT PM_DTScope_QuickRange_ck  CHECK (QuickRange IN ('Y','N')),
    Sundays          CHAR(1)       DEFAULT 'Y' NOT NULL,
      CONSTRAINT PM_DTScope_Sundays_ck  CHECK (Sundays IN ('Y','N')),
    Mondays          CHAR(1)       DEFAULT 'Y' NOT NULL,
      CONSTRAINT PM_DTScope_Mondays_ck  CHECK (Mondays IN ('Y','N')),
    Tuesdays         CHAR(1)       DEFAULT 'Y' NOT NULL,
      CONSTRAINT PM_DTScope_Tuesdays_ck  CHECK (Tuesdays IN ('Y','N')),
    Wednesdays       CHAR(1)       DEFAULT 'Y' NOT NULL,
      CONSTRAINT PM_DTScope_Wednesdays_ck  CHECK (Wednesdays IN ('Y','N')),
    Thursdays        CHAR(1)       DEFAULT 'Y' NOT NULL,
      CONSTRAINT PM_DTScope_Thursdays_ck  CHECK (Thursdays IN ('Y','N')),
    Fridays          CHAR(1)       DEFAULT 'Y' NOT NULL,
      CONSTRAINT PM_DTScope_Fridays_ck  CHECK (Fridays IN ('Y','N')),
    Saturdays        CHAR(1)       DEFAULT 'Y' NOT NULL,
      CONSTRAINT PM_DTScope_Saturdays_ck  CHECK (Saturdays IN ('Y','N')),
    tzregion         VARCHAR(64)   DEFAULT NULL,
    --
    -- check 
    --
    CONSTRAINT CHK_DTScope_DTPeriod CHECK ( DTPeriod >= 24 ),
    --
    -- primary key
    --
    CONSTRAINT PK_PM_DTScope PRIMARY KEY(Id)
)
TABLESPACE WM_Client;
--
-- sequence for PM_DTScope.Id
--
CREATE SEQUENCE SQ_DTScope
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE
;

--
-- Trigger from PM_DTScope to sync buttons and fields.
--
CREATE OR REPLACE TRIGGER TR_PM_DTScope_BIUR
  BEFORE INSERT OR UPDATE OF IncludeWeekends, WeekendsOnly ON PM_DTScope FOR EACH ROW
BEGIN
  IF ( :NEW.IncludeWeekends = 'N' ) THEN
     :NEW.Sundays := 'N';
     :NEW.Saturdays := 'N';
  END IF;
  IF ( :NEW.WeekendsOnly = 'Y' ) THEN
     :NEW.Mondays := 'N';
     :NEW.Tuesdays := 'N';
     :NEW.Wednesdays := 'N';
     :NEW.Thursdays := 'N';
     :NEW.Fridays := 'N';
  END IF;
  IF UPDATING THEN 
     IF ( ( UPDATING( 'INCLUDEWEEKENDS' ) OR UPDATING( 'WEEKENDSONLY' )  ) AND 
          NOT ( UPDATING( 'SUNDAYS' )    OR UPDATING( 'SATURDAYS' ) OR UPDATING( 'MONDAYS' ) OR UPDATING( 'TUESDAYS' ) OR
                UPDATING( 'WEDNESDAYS' ) OR UPDATING( 'THURSDAYS' ) OR UPDATING( 'FRIDAYS' ) ) ) THEN
  IF ( :NEW.IncludeWeekends = 'Y' ) THEN
     :NEW.Sundays := 'Y';
     :NEW.Saturdays := 'Y';
  END IF;
  IF ( :NEW.WeekendsOnly = 'N' ) THEN
     :NEW.Mondays := 'Y';
     :NEW.Tuesdays := 'Y';
     :NEW.Wednesdays := 'Y';
     :NEW.Thursdays := 'Y';
     :NEW.Fridays := 'Y';
  END IF;
     END IF;
  END IF;
  
END;
/
SHOW ERRORS TRIGGER TR_PM_DTScope_BIUR
  
  
CREATE TABLE PM_Calendar(
    Id               INTEGER       NOT NULL,
    DTScopeId        INTEGER       NOT NULL, -- parent object id
    CalendarDate     DATE,
--  FirstHour        SMALLINT      DEFAULT 1 CHECK (FirstHour Between 1 And 24) NOT NULL,
--  LastHour         SMALLINT      DEFAULT 24 CHECK (LastHour Between 1 And 24) NOT NULL,
    FirstHour        SMALLINT      DEFAULT 1 NOT NULL,
    LastHour         SMALLINT      DEFAULT 1 NOT NULL,
    FirstSeconds   INTEGER     DEFAULT 0 NOT NULL,
    LastSeconds    INTEGER       DEFAULT 0 NOT NULL,
    --
    -- foreign key(s)
    --
    CONSTRAINT FK_PM_Calendar_DTScopeId
    FOREIGN KEY(DTScopeId)         REFERENCES PM_DTScope(Id) ON DELETE CASCADE,
    --
    -- primary key
    --
    CONSTRAINT PK_PM_Calendar PRIMARY KEY(Id)
)
TABLESPACE WM_Client;
--
-- sequence for PM_Calendar.Id
--
CREATE SEQUENCE SQ_Calendar
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE
;


CREATE TABLE PM_Scenario(
    Id               INTEGER       NOT NULL,
    FolderId         INTEGER       NOT NULL,
    UserId           INTEGER       NOT NULL,
    Name             VARCHAR2(64)  NOT NULL,
    Description      VARCHAR2(255),
    Changed          DATE          DEFAULT TRUNC(SYSDATE,'MI') NOT NULL,
    --
    -- Type:
    --   'A' - ACTIVE scenario
    --   'C' - COMPLETE scenario
    --   'U' - USER scenario
    --
    Type              CHAR(1)       DEFAULT 'U' NOT NULL,
      CONSTRAINT PM_Scenario_Type_ck CHECK (Type IN ('A','C','U')),
    SystemId          INTEGER,      -- NULL for user scenarios, system id for active/complete
    Query_Id          INTEGER,      -- allow NULL so scenario can be created before query
    Active_Filter     CHAR(1)       DEFAULT 'N' NOT NULL,
      CONSTRAINT PM_Scenario_Active_Filter_ck CHECK (Active_Filter IN ('N','Y')),
    --
    -- foreign key(s)
    --
    CONSTRAINT FK_Scenario_Query_Id 
    FOREIGN KEY(Query_Id)             REFERENCES WMQ_QUERY(Query_Id),
    --
    -- constraints required to prevent removing folder or user
    --
    CONSTRAINT FK_Scenario_FolderId
    FOREIGN KEY(FolderId)          REFERENCES PM_Folder(Id),
    CONSTRAINT FK_Scenario_UserId
    FOREIGN KEY(UserId)            REFERENCES PM_User(Id),
    --
    -- Protect the scenario name
    --
    CONSTRAINT scenario_name_CK  CHECK( INSTR( TRANSLATE( name, '\/:;*?"''<>|', '???????????' ), '?' ) = 0),
    --
    -- primary key
    --
    CONSTRAINT PK_PM_Scenario PRIMARY KEY(Id)
)
TABLESPACE WM_Client;
--
-- index required to prevent duplicate documents in folder
--
CREATE UNIQUE INDEX UNQ_Scenario ON PM_Scenario(FolderId, UPPER(Name))
TABLESPACE WM_Client;
--
-- SEAde61762 Resolve deadlocks and invalid rollback
CREATE INDEX pm_scenario_query_id_idx ON pm_scenario(query_id)
  TABLESPACE wm_client;
--
-- sequence for PM_Scenario.Id
--
CREATE SEQUENCE SQ_Scenario
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE
;

--
-- support for expression-based scenarios
--
CREATE TABLE PM_Scenario_Filters(
    Scenario_Id      INTEGER          NOT NULL,
    Entity_Id        INTEGER          NOT NULL,
    Filter_Row       INTEGER          NOT NULL,
    Filter_Source    VARCHAR2(4000)   NOT NULL,
    Calc_Source      VARCHAR2(4000)   ,
    --
    -- foreign key(s)
    --
    CONSTRAINT FK_ScFilter_Scenario_Id
               FOREIGN KEY(Scenario_Id)          REFERENCES PM_Scenario(Id) ON DELETE CASCADE,
    CONSTRAINT FK_ScFilter_Entity_Id
               FOREIGN KEY(Entity_ID)            REFERENCES WMN_Entity(Entity_Id),
    --
    -- primary key
    --
    CONSTRAINT PK_PM_Scenario_Filters PRIMARY KEY(Scenario_Id, Entity_Id, Filter_Row)
)
TABLESPACE WM_Client;

COMMENT ON TABLE PM_Scenario_Filters IS 'Scenario Filter (create_client_schema.sql), Version 1.124.0';


CREATE TABLE PM_BusyHour (
    Id                  INTEGER       NOT NULL,
    UserId              INTEGER,
    Name                VARCHAR2(128)  NOT NULL,
    Changed             DATE          DEFAULT TRUNC(SYSDATE,'MI') NOT NULL,
    Cluster_Entity_Id   INTEGER       NOT NULL,
    Field_Id            INTEGER,      -- NULL for summation
    Description         VARCHAR2(255),
    FirstNeeded         DATE          DEFAULT TRUNC(SYSDATE,'MI') NOT NULL,
    --
    -- Span (TODO: server should support this in the near future)
    --   'D' - Daily
    --   'W' - Weekly
    --   'M' - Monthly
    --
    Span                CHAR(1)       DEFAULT 'D' NOT NULL,
--  MinRank             INTEGER       DEFAULT 1 CHECK (MinRank Between 1 And 24) NOT NULL,
--  MaxRank             INTEGER       DEFAULT 1 CHECK (MaxRank Between 1 And 24) NOT NULL,
    MinRank             INTEGER       DEFAULT 1 NOT NULL,
    MaxRank             INTEGER       DEFAULT 1 NOT NULL,
    FirstAvail          DATE, 
    LastAvail           DATE,
    DTPeriod    INTEGER       DEFAULT 24 NOT NULL,
    scenario_id         INTEGER,
    daily_retention     INTEGER,
    weekly_retention    INTEGER,
    monthly_retention   INTEGER,
    -- 
    -- check 
    --
    CONSTRAINT CHK_BusyHour_DTPeriod CHECK ( DTPeriod >= 24 ),
    --
    -- foreign key(s)
    --
    CONSTRAINT FK_BusyHour_Cluster_Ent_Id
         FOREIGN KEY(Cluster_Entity_Id) REFERENCES WMN_ENTITY(Entity_Id),
    CONSTRAINT FK_BusyHour_Field_Id 
         FOREIGN KEY(Field_Id)          REFERENCES WMD_FIELD_DEF(Field_Id),
    CONSTRAINT FK_BusyHour_Scenario_Id 
         FOREIGN KEY(scenario_id)       REFERENCES PM_SCENARIO(Id),
    --
    -- primary key
    --
    CONSTRAINT PK_PM_BusyHour PRIMARY KEY(Id)
)
TABLESPACE WM_Client;
--
-- sequence for PM_BusyHour.Id
--
CREATE SEQUENCE SQ_BusyHour
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE
;

CREATE UNIQUE INDEX UNQ_BusyHour ON PM_BusyHour(
  Cluster_Entity_Id, DECODE(Field_Id,NULL,NULL,UPPER(Name)),span
)
TABLESPACE WM_Client;

CREATE TABLE PM_HourRange(
    Id               INTEGER       NOT NULL,
    BusyHourId       INTEGER       NOT NULL, -- parent object id
    FirstHour        SMALLINT      DEFAULT 1, -- CHECK (FirstHour Between 1 And 24) NOT NULL,
    LastHour         SMALLINT      DEFAULT 1, -- CHECK (LastHour Between 1 And 24) NOT NULL,
    FirstSeconds     INTEGER       DEFAULT 0 NOT NULL,
    LastSeconds      INTEGER       DEFAULT 0 NOT NULL,
    --
    -- foreign key(s)
    --
    CONSTRAINT FK_HourRange_BusyHourId
  FOREIGN KEY(BusyHourId)        REFERENCES PM_BusyHour(Id) ON DELETE CASCADE,
  --
  -- unique constraint
  --
  CONSTRAINT pm_hourrange_uq UNIQUE (busyhourid,firsthour,lasthour) using INDEX TABLESPACE wm_client,
    --
    -- primary key
    --
    CONSTRAINT PK_PM_HourRange PRIMARY KEY(Id)
)
TABLESPACE WM_Client;
--
--
-- sequence for PM_HourRange.Id
--
CREATE SEQUENCE SQ_HourRange
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE
;

--------------------------------------------------------------------------------
-- PM_PABH. Sibling table to PM_BusyHour. Shared primary key.
--------------------------------------------------------------------------------
CREATE TABLE PM_PABH (
    Id                  INTEGER       NOT NULL,
    FirstAvail          DATE, 
    LastAvail           DATE,
    CONSTRAINT FK_PM_PABH_Id
       FOREIGN KEY (Id)       REFERENCES PM_BusyHour(Id)
       ON DELETE CASCADE,
    CONSTRAINT PK_PM_PABH
       PRIMARY KEY (Id)
)
TABLESPACE WM_Client;


CREATE TABLE PM_ChanSetPlan(
    Id               INTEGER       NOT NULL,
    FolderId         INTEGER       NOT NULL,
    UserId           INTEGER       NOT NULL,
    Name             VARCHAR2(64)  NOT NULL,
    Description      VARCHAR2(255),
    Changed          DATE          DEFAULT TRUNC(SYSDATE,'MI') NOT NULL,
    --
    -- constraints required to prevent removing folder or user
    --
    CONSTRAINT FK_ChanSetPlan_FolerId
    FOREIGN KEY(FolderId)          REFERENCES PM_Folder(Id),
    CONSTRAINT FK_ChanSetPlan_UserId
    FOREIGN KEY(UserId)            REFERENCES PM_User(Id),
    --
    -- primary key
    --
    CONSTRAINT PK_PM_ChanSetPlan PRIMARY KEY(Id)
)
TABLESPACE WM_Client;
--
-- index required to prevent duplicate documents in folder
--
CREATE UNIQUE INDEX UNQ_ChanSetPlan ON PM_ChanSetPlan(FolderId, UPPER(Name))
TABLESPACE WM_Client;
--
-- sequence for PM_ChanSetPlan.Id
--
CREATE SEQUENCE SQ_ChanSetPlan
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE
;

CREATE TABLE PM_Partition(
    ChanSetPlanId    INTEGER       NOT NULL,
    Ordinal          INTEGER       NOT NULL,
    Name             VARCHAR2(64)  NOT NULL,
    NumberOfRows     INTEGER       NOT NULL,
    IsSetup          CHAR(1)       DEFAULT 'N' NOT NULL,
      CONSTRAINT PM_Partition_IsSetup_ck CHECK (IsSetup IN ('Y','N')),
    --
    -- foreign key(s)
    --
    CONSTRAINT FK_Partition_ChanSetPlanId
    FOREIGN KEY(ChanSetPlanId)     REFERENCES PM_ChanSetPlan(Id) ON DELETE CASCADE,
    --
    -- primary key
    --
    CONSTRAINT PK_PM_Partition PRIMARY KEY(ChanSetPlanId, Ordinal)
)
TABLESPACE WM_Client;

CREATE TABLE PM_ChanGrp(
    ChanSetPlanId    INTEGER       NOT NULL,
    Ordinal          INTEGER       NOT NULL,
    Name             VARCHAR2(64)  NOT NULL,
    BackColor        INTEGER       DEFAULT 16777215 NOT NULL, -- default: white
    --
    -- foreign key(s)
    --
    CONSTRAINT FK_ChanGrp_ChanSetPlanId
    FOREIGN KEY(ChanSetPlanId)     REFERENCES PM_ChanSetPlan(Id) ON DELETE CASCADE,
    --
    -- primary key
    --
    CONSTRAINT PK_PM_ChanGrp PRIMARY KEY(ChanSetPlanId, Ordinal)
)
TABLESPACE WM_Client;

CREATE TABLE PM_ChanSetGrp(
    ChanSetPlanId    INTEGER       NOT NULL,
    Ordinal          INTEGER       NOT NULL,
    Name             VARCHAR2(64)  NOT NULL,
    BackColor        INTEGER       DEFAULT 16777215 NOT NULL, -- default: white
    --
    -- foreign key(s)
    --
    CONSTRAINT FK_ChanSetGrp_ChanSetPlanId
    FOREIGN KEY(ChanSetPlanId)     REFERENCES PM_ChanSetPlan(Id) ON DELETE CASCADE,
    --
    -- primary key
    --
    CONSTRAINT PK_PM_ChanSetGrp PRIMARY KEY(ChanSetPlanId, Ordinal)
)
TABLESPACE WM_Client;

CREATE TABLE PM_ChanSet(
    ChanSetPlanId    INTEGER       NOT NULL,
    Ordinal          INTEGER       NOT NULL,
    Name             VARCHAR2(64)  NOT NULL,
    ChanSetGrpName   VARCHAR2(64)  NOT NULL,
    --
    -- foreign key(s)
    --
    CONSTRAINT FK_ChanSet_ChanSetPlanId
    FOREIGN KEY(ChanSetPlanId)     REFERENCES PM_ChanSetPlan(Id) ON DELETE CASCADE,
    --
    -- primary key
    --
    CONSTRAINT PK_PM_ChanSet PRIMARY KEY(ChanSetPlanId, Ordinal)
)
TABLESPACE WM_Client;

CREATE TABLE PM_Channel(
    ChanSetPlanId    INTEGER       NOT NULL,
    Ordinal          INTEGER       NOT NULL,
    ChanSetName      VARCHAR2(64)  NOT NULL,
    ChanGrpName      VARCHAR2(64)  NOT NULL,
    PartitionName    VARCHAR2(64)  NOT NULL,
    RowNumber        INTEGER       NOT NULL,
    --
    -- foreign key(s)
    --
    CONSTRAINT FK_Channel_ChanSetPlanId
    FOREIGN KEY(ChanSetPlanId)     REFERENCES PM_ChanSetPlan(Id) ON DELETE CASCADE,
    --
    -- primary key
    --
    CONSTRAINT PK_PM_Channel PRIMARY KEY(ChanSetPlanId, Ordinal)
)
TABLESPACE WM_Client;

CREATE TABLE PM_CfgTpl(
    Id               INTEGER       NOT NULL,
    FolderId         INTEGER       NOT NULL,
    UserId           INTEGER       NOT NULL,
    Name             VARCHAR2(64)  NOT NULL,
    Description      VARCHAR2(255),
    Changed          DATE          DEFAULT TRUNC(SYSDATE,'MI') NOT NULL,
    Category         VARCHAR2(64)  NOT NULL,
    Type             VARCHAR2(64)  NOT NULL,
    Entity_Id        INTEGER,      -- NULL for scenario and channel use based templates
    NeedsInput       INTEGER       NOT NULL,
    --
    -- constraints required to prevent removing folder or user
    --
    CONSTRAINT FK_PM_CFGTPL_FOLDERID
    FOREIGN KEY(FolderId)          REFERENCES PM_Folder(Id),
    CONSTRAINT FK_PM_CFGTPL_USERID
    FOREIGN KEY(UserId)            REFERENCES PM_User(Id),
    CONSTRAINT FK_PM_CFGTPL_ENTITY_ID
    FOREIGN KEY(Entity_Id)         REFERENCES WMN_ENTITY(Entity_Id),
    --
    -- primary key
    --
    CONSTRAINT PK_PM_CfgTpl PRIMARY KEY(Id)
)
TABLESPACE WM_Client;
--
-- index required to prevent duplicate documents in folder
--
CREATE UNIQUE INDEX UNQ_CfgTpl ON PM_CfgTpl(FolderId, UPPER(Name), Category, Type)
TABLESPACE WM_Client;

CREATE TABLE PM_Template(
    Id               INTEGER       NOT NULL,
    FolderId         INTEGER       NOT NULL,
    UserId           INTEGER       NOT NULL,
    Name             VARCHAR2(64)  NOT NULL,
    Description      VARCHAR2(255),
    Changed          DATE          DEFAULT TRUNC(SYSDATE,'MI') NOT NULL,
    Category         VARCHAR2(64)  NOT NULL,
    Type             VARCHAR2(64)  NOT NULL,
    Entity_Id        INTEGER,      -- NULL for UDR, CPFail Count/Message , PCMD Count/Message
    CriteriaFieldId  INTEGER,      -- note: not populated by client yet
    --
    -- constraints required to prevent removing folder or user
    --
    CONSTRAINT FK_Temp_FolderId
    FOREIGN KEY(FolderId)          REFERENCES PM_Folder(Id),
    CONSTRAINT FK_Temp_UserId
    FOREIGN KEY(UserId)            REFERENCES PM_User(Id),
    CONSTRAINT FK_Temp_Entity_Id
    FOREIGN KEY(Entity_Id)         REFERENCES WMN_ENTITY(Entity_Id),
    CONSTRAINT FK_Temp_CriteriaFieldId
    FOREIGN KEY(CriteriaFieldId)   REFERENCES WMD_FIELD_DEF(Field_Id),
    --
    -- primary key
    --
    CONSTRAINT PK_PM_Template PRIMARY KEY(Id)
)
TABLESPACE WM_Client;
--
-- index required to prevent duplicate documents in folder
--
CREATE UNIQUE INDEX UNQ_Template ON PM_Template(FolderId, UPPER(Name), Category, Entity_ID)
TABLESPACE WM_Client;
--
-- sequence for PM_Template.Id
--
CREATE SEQUENCE SQ_Template
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE
;

CREATE TABLE PM_TemplateField(
    TemplateId       INTEGER       NOT NULL, -- parent object id
    Ordinal          INTEGER       NOT NULL,
    Field_Id         INTEGER       NOT NULL,
    --
    -- SortOrder:
    --   'N' - No sorting
    --   'A' - Ascending
    --   'D' - Descending
    --
    SortOrder        CHAR(1)       DEFAULT 'N' NOT NULL,
      CONSTRAINT PM_TemplateField_SortOrder_ck CHECK (SortOrder IN ('N','A','D')),
    IsShown          CHAR(1)       DEFAULT 'Y' NOT NULL, 
      CONSTRAINT PM_TemplateField_IsShown_ck CHECK (IsShown IN ('Y','N')),
    -- 
    -- support for Field Attribute Values in Reports (SEAde40358)
    --
    comment_attr_name VARCHAR2(32),
    --
    -- foreign key(s)
    --
    CONSTRAINT FK_Temp_Fld_TemplateId
    FOREIGN KEY(TemplateId)        REFERENCES PM_Template(Id) ON DELETE CASCADE,
    CONSTRAINT FK_Temp_fld_Field_Id
    FOREIGN KEY(Field_Id)          REFERENCES WMD_FIELD_DEF(Field_Id),
    --
    -- primary key
    --
    CONSTRAINT PK_PM_TemplateField PRIMARY KEY(TemplateId, Ordinal)
)
TABLESPACE WM_Client;

CREATE TABLE PM_Filter(
    TemplateId       INTEGER       NOT NULL, -- parent object id
    FieldOrdinal     INTEGER       NOT NULL,
    Ordinal          INTEGER       NOT NULL,
    Criteria         VARCHAR2(2048),
    --
    -- foreign key(s)
    --
    CONSTRAINT FK_Filter_TempId_FldOrd
    FOREIGN KEY(TemplateId, FieldOrdinal) 
      REFERENCES PM_TemplateField(TemplateId, Ordinal) ON DELETE CASCADE,
    --
    -- primary key
    --
    CONSTRAINT PK_PM_Filter PRIMARY KEY(TemplateId, FieldOrdinal, Ordinal)
)
TABLESPACE WM_Client;

CREATE TABLE PM_CPFailCountType(
    Id               INTEGER       NOT NULL,
    Name             VARCHAR2(64)  NOT NULL,
    UsesDN           CHAR(1)       NOT NULL,
      CONSTRAINT PM_CPFailCountType_UsesDN_ck CHECK (UsesDN IN ('Y','N')),
    UsesThreshold    CHAR(1)       NOT NULL,
      CONSTRAINT PM_CPFailCountType_UsesThre_ck CHECK (UsesThreshold IN ('Y','N')),
    -- SEAde35467
    CountType        VARCHAR2(64)  NOT NULL,
      CONSTRAINT PM_CPFailCountType_CountTyp_ck CHECK (CountType IN ('CPFail','PCMD')),
    --
    -- primary key
    --
    CONSTRAINT PK_PM_CPFailCountType PRIMARY KEY(Id)
)
TABLESPACE WM_Client;

CREATE TABLE PM_GroupBy(
    Id              INTEGER        NOT NULL,
    GroupBy         VARCHAR2(64)   NOT NULL,
    GroupUsing      CHAR(1)        DEFAULT 'Y' NOT NULL,
      CONSTRAINT PM_GroupBy_GroupUsing_ck CHECK (GroupUsing IN ('Y','N')),
    Forecast        CHAR(1)        DEFAULT 'N' NOT NULL,
      CONSTRAINT PM_GroupBy_Forecast_ck  CHECK (Forecast IN ('N','Y')),
    -- frequency_id now used to support subhourly repgen:
    frequency_id    INTEGER,
    orientation     CHAR(1),
    matrix          CHAR(1)        DEFAULT 'N' NOT NULL,
      CONSTRAINT PM_GroupBy_matrix_ck  CHECK (matrix IN ('N','Y')),
    confidence      CHAR(1)        DEFAULT 'N' NOT NULL, 
      CONSTRAINT PM_GroupBy_confidence_ck  CHECK (confidence IN ('N', 'Y')),
    display_order   INTEGER        DEFAULT 0 NOT NULL,
    CONSTRAINT FK_pm_groupby_fid FOREIGN KEY (frequency_id) 
  REFERENCES wm_frequency(frequency_id),
    startofweek     CHAR(1)  DEFAULT 'N' NOT NULL,
    CONSTRAINT  pm_grp_sow_chk CHECK ( startofweek IN ('Y', 'N' )),  
    --
    -- primary key
    --
    CONSTRAINT PK_PM_GroupBy PRIMARY KEY(Id)
)
TABLESPACE WM_Client;

Create unique index unq_groupby_idx on pm_groupby(upper(groupby));


CREATE TABLE PM_DimensionType
(
   Id               INTEGER        NOT NULL,
   Name             VARCHAR2(16)   NOT NULL,
    --
    -- primary key
    --
   CONSTRAINT PK_PM_DimensionType PRIMARY KEY(Id)
)
TABLESPACE WM_Client;

-- SEAde40596: 
-- for GroupByTemplateField feature
-- start
CREATE TABLE PM_GROUPINGSPEC
(
   ID            INTEGER NOT NULL,
   TIME_FIRST    CHAR(1) DEFAULT 'N' NOT NULL,
   YEAR          CHAR(1) DEFAULT 'N' NOT NULL,
   QUARTER       CHAR(1) DEFAULT 'N' NOT NULL,
   MONTH         CHAR(1) DEFAULT 'N' NOT NULL,
   WEEK          CHAR(1) DEFAULT 'N' NOT NULL,
   DAY           CHAR(1) DEFAULT 'N' NOT NULL,
   HOUR          CHAR(1) DEFAULT 'N' NOT NULL,
   MIN30         CHAR(1) DEFAULT 'N' NOT NULL,
   MIN15         CHAR(1) DEFAULT 'N' NOT NULL,
   MIN10         CHAR(1) DEFAULT 'N' NOT NULL,
   MIN5          CHAR(1) DEFAULT 'N' NOT NULL,
   MIN2          CHAR(1) DEFAULT 'N' NOT NULL,
   MIN1          CHAR(1) DEFAULT 'N' NOT NULL,
   -- primary key
   CONSTRAINT PK_PM_GROUPINGSPEC  PRIMARY KEY( ID ),
   -- check constraints
   CONSTRAINT CK_TIME_FIRST CHECK( TIME_FIRST IN ('Y','N' ) ),
   CONSTRAINT CK_YEAR    CHECK( YEAR    IN ('Y','N' ) ),
   CONSTRAINT CK_QUARTER CHECK( QUARTER IN ('Y','N' ) ),
   CONSTRAINT CK_MONTH   CHECK( MONTH   IN ('Y','N' ) ),
   CONSTRAINT CK_WEEK    CHECK( WEEK    IN ('Y','N' ) ),
   CONSTRAINT CK_DAY     CHECK( DAY     IN ('Y','N' ) ),
   CONSTRAINT CK_HOUR    CHECK( HOUR    IN ('Y','N' ) ),
   CONSTRAINT CK_MIN30   CHECK( MIN30   IN ('Y','N' ) ),
   CONSTRAINT CK_MIN15   CHECK( MIN15   IN ('Y','N' ) ),
   CONSTRAINT CK_MIN10   CHECK( MIN10   IN ('Y','N' ) ),
   CONSTRAINT CK_MIN5    CHECK( MIN5    IN ('Y','N' ) ),
   CONSTRAINT CK_MIN2    CHECK( MIN2    IN ('Y','N' ) ),
   CONSTRAINT CK_MIN1    CHECK( MIN1    IN ('Y','N' ) )
)
TABLESPACE WM_CLIENT;

CREATE SEQUENCE PM_SEQ_GROUPINGSPEC;

CREATE TABLE PM_REPORTGROUPING
(
   GROUPINGSPEC_ID INTEGER NOT NULL,
   ORDINAL      INTEGER NOT NULL,
   FIELD_ID     INTEGER NOT NULL,
   -- primary key   
   CONSTRAINT PK_PM_REPORTGROUPING 
              PRIMARY KEY( GROUPINGSPEC_ID, ORDINAL ),
   -- foriegn key for spec.
   CONSTRAINT FK_PM_RPTGRPING_GROUPSPEC_ID
              FOREIGN KEY(groupingspec_id) REFERENCES PM_GROUPINGSPEC(id)
              ON DELETE CASCADE,
   -- foreign key for field.
   CONSTRAINT FK_PM_RPTGRPING_FIELD_ID 
              FOREIGN KEY(field_id) REFERENCES  WMD_FIELD_DEF(field_id)
              ON DELETE CASCADE
)
TABLESPACE WM_CLIENT;
--
-- end: SEAde40596

CREATE TABLE PM_ReportSpec(
    Id               INTEGER       NOT NULL,
    --
    -- ReportKind:
    --   'S' - Standard (Traffic)
    --   'H' - N-High Hours (Traffic)
    --   'D' - N-High Days (Traffic)
    --   'C' - CPFail Counts (CPFail)
    --   'M' - CPFail Message (CPFail)
    --   'O' - CPFail Handoff (CPFail)
    --   'N' - Normalized Busy Hour (Traffic)
    --   'P' - Percentile Busy Hour (Traffic)
    --   'I' - For Lucent PCMD Message All report. I means Include CFC=1 or 
    --   'E' - For Lucent PCMD Message Fail report. E means Exclude CFC=1 
    --   'R' - for Lucent PCMD Message Failure Rate Summary Report
    ReportKind       CHAR(1)       NOT NULL,
      CONSTRAINT PM_ReportSpec_ReportKind_ck CHECK (ReportKind IN ('S','H','D','C','M','G','F', 'A', 'O', 'N', 'I', 'E', 'P', 'Z', 'R' )),
    --
    DoIndividual     CHAR(1)       DEFAULT 'N' NOT NULL, 
      CONSTRAINT PM_ReportSpec_DoIndividual_ck CHECK (DoIndividual IN ('Y','N')),
    --
    DoSummary        CHAR(1)       DEFAULT 'Y' NOT NULL,
      CONSTRAINT PM_ReportSpec_DoSummary_ck CHECK (DoSummary IN ('Y','N')),
    --
    -- SortTypes:
    --   'A' - Row ascending
    --   'D' - Row descending
    --   'T' - Template sort order
    --   'B' - Busy Time ascending
    --   'U' - Busy Time descending
    --
    IndSortType      CHAR(1)       DEFAULT 'A' NOT NULL, 
      CONSTRAINT PM_ReportSpec_IndSortType_ck CHECK (IndSortType IN ('A','D','T','B','U')),
    --
    SumSortType      CHAR(1)       DEFAULT 'A' NOT NULL, 
      CONSTRAINT PM_ReportSpec_SumSortType_ck CHECK (SumSortType IN ('A','D','T')),
    --
    --
    -- GraphTypes:
    --   'N' - None
    --   'C' - Column
    --   'L' - Line
    --   'P' - Pie
    --   'X' - XY
    --
    IndGraphType     CHAR(1)       DEFAULT 'N' NOT NULL,
      CONSTRAINT PM_ReportSpec_IndGraphType_ck CHECK (IndGraphType IN ('N','C','L','P','X', '1')),
    SumGraphType     CHAR(1)       DEFAULT 'N' NOT NULL,
      CONSTRAINT PM_ReportSpec_SumGraphType_ck CHECK (SumGraphType IN ('N','C','L','P','X', '1')),
    DoStored         CHAR(1)       DEFAULT 'N' NOT NULL,  -- for Traffic, NULL otherwise
      CONSTRAINT PM_ReportSpec_DoStored_ck CHECK (DoStored IN ('Y','N')),
    DoMissingData    CHAR(1)       DEFAULT 'N' NOT NULL,
      CONSTRAINT PM_ReportSpec_DoMissingData_ck CHECK (DoMissingData IN ('Y','N')),
    GroupById        INTEGER,      -- sub-object id (used if DoStored='Y', NULL otherwise)
    RangeMin         INTEGER,      -- for Standard or N-High, NULL otherwise
    RangeMax         INTEGER,      -- for Standard or N-High, NULL otherwise
    BusyHourId       INTEGER,      -- sub-object id (used if ReportKind='S', NULL otherwise)
    -- Also used for PCMD
    CPFailCountTypeId   INTEGER,   -- sub-object id (used if ReportKind='C', NULL otherwise)
    Cluster_Entity_Id   INTEGER,   -- NULL for CPFail Count/Message, Traffic Summation
    Field_Id            INTEGER,
    DN               VARCHAR2(32), -- for Cell Failures by DN, NULL otherwise
    Threshold        INTEGER,      -- for CPFail Counts, NULL otherwise
    GraphId          INTEGER,
    mapid            NUMBER,
    dimensiontypeid  INTEGER,
    forecastthreshold    FLOAT,
    currentthreshold     FLOAT,
    finalthreshold       FLOAT,
    correlationthreshold FLOAT,
    n1days       INTEGER,
    n2days       INTEGER,
    n3days       INTEGER,
    bhscenario_id    INTEGER,
    --
    -- foreign key(s)
    --
    CONSTRAINT FK_RptSpec_GroupById
    FOREIGN KEY(GroupById)          REFERENCES PM_GroupBy(Id),
    CONSTRAINT FK_Rptspec_BusyHourId
    FOREIGN KEY(BusyHourId)         REFERENCES PM_BusyHour(Id),
    CONSTRAINT FK_RptSpec_CpFailCountTypeId
    FOREIGN KEY(CPFailCountTypeId)  REFERENCES PM_CPFailCountType(Id),
    CONSTRAINT FK_RptSpec_Cluster_Ent_Id
    FOREIGN KEY(Cluster_Entity_Id)  REFERENCES WMN_ENTITY(Entity_Id),
    CONSTRAINT FK_RptSpec_Field_Id
    FOREIGN KEY(Field_Id)           REFERENCES WMD_FIELD_DEF(Field_Id),
    CONSTRAINT FK_PM_DimensionTypeId
                FOREIGN KEY(dimensiontypeid)    REFERENCES PM_DimensionType(Id),
    CONSTRAINT FK_BHScenario_Id 
    FOREIGN KEY(bhscenario_id)      REFERENCES PM_SCENARIO(Id),
    --
    -- primary key
    --
    CONSTRAINT PK_PM_ReportSpec PRIMARY KEY(Id)
)
TABLESPACE WM_Client;
--
--SEAde61762 Resolve deadlocks and invalid rollback
CREATE INDEX pm_reportspec_bhscen_id_idx ON PM_Reportspec(bhscenario_id)
  TABLESPACE wm_client;
--
-- sequence for PM_ReportSpec.Id
--
CREATE SEQUENCE SQ_ReportSpec
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE
;

CREATE TABLE PM_Report(
    Id               INTEGER       NOT NULL,
    FolderId         INTEGER       NOT NULL,
    UserId           INTEGER       NOT NULL,
    Name             VARCHAR2(100) NOT NULL,
    Description      VARCHAR2(255),
    Changed          DATE          DEFAULT TRUNC(SYSDATE,'MI') NOT NULL,
    Category         VARCHAR2(64)  NOT NULL,
    Type             VARCHAR2(64)  NOT NULL,
    Entity_Id        INTEGER,      -- NULL for Config, UDR, CPFail Count/Message, Enterprise reports
    ScenarioId       INTEGER,      -- folder document id. NULL for Enterprise reports
    TemplateId       INTEGER,      -- folder document id (Template or CfgTpl). NULL for Enterprise reports
    ChanSetPlanId    INTEGER,                -- folder document id
    DestinationId    INTEGER       NOT NULL, -- sub-object id
    PageSetupId      INTEGER,                -- sub-object id
    DTScopeId        INTEGER,                -- sub-object id
    ReportSpecId     INTEGER,                -- sub-object id
    PE_ReportSpecId  INTEGER,      -- sub-object id for Prospect Enterprise (traffic cache)
    ChannelNumber    INTEGER,      -- NULL except for Config reports requiring channel number
    DomainScenarioId INTEGER,      -- NULL except for 'Co-channel Usage Report' and 'Adjacent Channel Usage Report' Config Entity Reports 
    MapRadiusScale   NUMBER,       -- NULL except for Config Map Reports
    TestChannels     CHAR(1),      -- NULL except for 'Co-channel Usage Report' and 'Adjacent Channel Usage Report' Config Entity Reports 
    EquippedOnly     CHAR(1),      -- NULL except for 'Co-channel Usage Report' and 'Adjacent Channel Usage Report' Config Entity Reports
    FileName         VARCHAR2(255),
    GroupingSpec_Id  INTEGER,
    --
    -- foreign key(s)
    --
    CONSTRAINT FK_Report_DestinationId
    FOREIGN KEY(DestinationId)     REFERENCES PM_Destination(Id),
    CONSTRAINT FK_Report_PageSetupId
    FOREIGN KEY(PageSetupId)       REFERENCES PM_PageSetup(Id),
    CONSTRAINT FK_Report_DTScopeId
    FOREIGN KEY(DTScopeId)         REFERENCES PM_DTScope(Id),
    CONSTRAINT FK_Report_ReportSpecId
    FOREIGN KEY(ReportSpecId)      REFERENCES PM_ReportSpec(Id),
    CONSTRAINT FK_Report_Entity_Id
    FOREIGN KEY(Entity_Id)         REFERENCES WMN_ENTITY(Entity_Id),
    --
    -- constraints required to prevent removing folder or user
    --
    CONSTRAINT FK_Report_FolderId
    FOREIGN KEY(FolderId)          REFERENCES PM_Folder(Id),
    CONSTRAINT FK_Report_UserId
    FOREIGN KEY(UserId)            REFERENCES PM_User(Id),
    --
    -- primary key
    --
    CONSTRAINT PK_PM_Report PRIMARY KEY(Id),
    --
    -- constraint for GROUPINGSPEC (SEAde40596)
    --
    CONSTRAINT FK_PM_REPORT_GROUPINGSPEC_ID
                FOREIGN KEY(GroupingSpec_Id)   REFERENCES PM_GROUPINGSPEC(id)
) TABLESPACE WM_Client;
--SEAde61762 Resolve deadlocks and invalid rollback
CREATE INDEX pm_report_pagesetupid_idx ON pm_report(pagesetupid)
  TABLESPACE wm_client;
--
PROMPT Creating trigger pm_report_del_tr...
-- SEAde15088: Client not deleting rows from PM_DTSCOPE.
-- SEAde43750: User can not delete a report from Prospect Client.
CREATE OR REPLACE TRIGGER pm_report_del_tr BEFORE DELETE ON pm_report FOR EACH ROW
DECLARE
   constraint_error EXCEPTION;
   PRAGMA EXCEPTION_INIT(constraint_error, -02292);
BEGIN
  DELETE FROM PM_DTScope WHERE ID = :OLD.DTScopeId;

  EXCEPTION
     WHEN constraint_error THEN
        -- Silently allow the delete to fail. It must be in use by RG_SCHEDULE
        NULL;
END;
/
SHOW ERRORS TRIGGER pm_report_del_tr

-- SEAde47897:  Prevent deadlocks.
CREATE INDEX pm_report_specid_idx ON pm_report( reportspecid ) TABLESPACE WM_Client;
-- SEAde64771:  Prevent deadlocks on dtscope
CREATE INDEX pm_report_dtscope_idx ON pm_report( dtscopeid ) TABLESPACE WM_Client;

--
-- index required to prevent duplicate documents in folder
--
CREATE UNIQUE INDEX UNQ_Report ON PM_Report(FolderId, UPPER(Name), Category, Entity_Id)
TABLESPACE WM_Client;
--
-- enforce 1 <-> 1 relationship on PM_Report <-> PM_Destination
--
CREATE UNIQUE INDEX UNQ_Report_Dest_Id ON PM_Report(DestinationId)
TABLESPACE WM_Client;
--
-- sequence for PM_Report.Id
--
CREATE SEQUENCE document_seq
    START WITH 10000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE
;

CREATE SYNONYM SQ_Report FOR document_seq;

--
--  Trigger to null scenario id in PM_Report when scenario is deleted
--
CREATE OR REPLACE TRIGGER PM_Scenario_TR_BD
  AFTER DELETE ON PM_Scenario
  FOR EACH ROW
BEGIN
  UPDATE PM_Report
    SET ScenarioID = 0
    WHERE scenarioid = :old.id;
END;
/
SHOW ERRORS
  

CREATE TABLE Job_Group(
    Id               INTEGER       NOT NULL,
    FolderId         INTEGER       NOT NULL,
    UserId           INTEGER       NOT NULL,
    Name             VARCHAR2(64)  NOT NULL,
    Description      VARCHAR2(255),
    Changed          DATE          DEFAULT TRUNC(SYSDATE,'MI') NOT NULL,
    Group_Type       INTEGER       NOT NULL,
    --
    -- constraints required to prevent removing folder or user
    --
    CONSTRAINT FK_Job_Group_FolderId
    FOREIGN KEY(FolderId)          REFERENCES PM_Folder(Id),
    CONSTRAINT FK_Job_Group_UserId
    FOREIGN KEY(UserId)            REFERENCES PM_User(Id),
    --
    -- primary key
    --
    CONSTRAINT PK_Job_Group PRIMARY KEY(Id)
)
TABLESPACE WM_Client;
--
-- index required to prevent duplicate documents in folder
--
--SEAde26640 allow same document name for documents of different group_type

CREATE UNIQUE INDEX UNQ_Job_Group ON Job_Group(FolderId, UPPER(Name),group_type)
TABLESPACE WM_Client;

--
-- sequence for PM_RptGrp.Id
--
CREATE SEQUENCE job_group_seq
    START WITH 10000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE ;

CREATE SYNONYM SQ_RptGrp FOR job_group_seq;

-- View to fool old code.
CREATE OR REPLACE FORCE VIEW  PM_RptGrp AS
  SELECT * FROM job_group;

CREATE TABLE job_group_item (
  id            NUMBER(38),
  group_id  NUMBER(38),
  document_id   NUMBER(38),
  ordinal       NUMBER(38), 
    CONSTRAINT job_group_item_pk PRIMARY KEY ( id ) USING INDEX TABLESPACE wm_client,
    CONSTRAINT job_group_item_uq UNIQUE (group_id, document_id ) USING INDEX TABLESPACE wm_client,
    CONSTRAINT job_group_item_group_id_fk FOREIGN KEY ( group_id ) REFERENCES job_group( id ) ON DELETE CASCADE
) TABLESPACE wm_client;

-- View to fool old code.
CREATE OR REPLACE FORCE VIEW PM_RptGrpItem AS
  SELECT id Id, group_id RptGrpId, document_id ReportId, ordinal Ordinal 
    FROM job_group_item;

CREATE SEQUENCE job_group_item_seq
    START WITH 10000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE;

CREATE SYNONYM SQ_RptGrpItem FOR job_group_item_seq;

--
-- create view of PM_RptGrp with join to PM_RptGrpItem to get count of reports
--
CREATE VIEW VW_RptGrp AS
SELECT
    g.Id,
    g.FolderId,
    g.UserId,
    g.Name,
    g.Description,
    g.Changed,
    COUNT(i.Id) RptCount
FROM
    Job_Group g,
    PM_RptGrpItem i 
WHERE
    g.Id = i.RptGrpId(+)
GROUP BY
    g.Id,
    g.FolderId,
    g.UserId,
    g.Name,
    g.Description,
    g.Changed;

--
--
--

-- Was: PM_Schedule

--
-- sequence for PM_Schedule.Id
--
CREATE SEQUENCE SQ_Schedule
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE
;

CREATE TABLE PM_Holidays(
    Holiday            DATE          NOT NULL,
    --
    -- primary key
    --
    CONSTRAINT PK_PM_Holidays PRIMARY KEY(Holiday)
)
TABLESPACE WM_Client;

CREATE TABLE PM_Trigger(
    Trigger_Id          INTEGER         NOT NULL,
    Trigger_Name        VARCHAR2(30)    NOT NULL,
    SubTrigger_Name     VARCHAR2(30)    NOT NULL,
    Template_Id         INTEGER         NOT NULL,
    --
    -- primary key
    --
    CONSTRAINT PK_PM_Trigger PRIMARY KEY(Trigger_Id)
    --
    -- foreign key(s)
    --
    -- FOREIGN KEY(Template_Id) REFERENCES PM_Template(Id)
)
TABLESPACE WM_Client;

--
-- sequence for PM_Trigger.Trigger_Id
--
CREATE SEQUENCE SQ_Trigger
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE
;

CREATE UNIQUE INDEX UNQ_PM_Trigger ON PM_Trigger(UPPER(Trigger_Name), UPPER(SubTrigger_Name), Template_Id)
TABLESPACE WM_Client;

CREATE TABLE PM_Feature(
    Feature_Id          INTEGER         NOT NULL,
    Feature_Name        VARCHAR2(30)    NOT NULL,
    --
    -- primary key
    --
    CONSTRAINT PK_PM_Feature PRIMARY KEY(Feature_Id)
)
TABLESPACE WM_Client;

CREATE SEQUENCE PM_Feature_SEQ
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE;

CREATE OR REPLACE Trigger PM_Feature_TR_BI
    Before Insert on PM_Feature
    For Each Row
BEGIN
    IF :New.Feature_Id is NULL
    THEN
       SELECT PM_Feature_SEQ.NextVal INTO :New.Feature_Id FROM DUAL;
    END IF;
END;
/

CREATE TABLE PM_Trend_Impact(
    Trigger_Id        INTEGER     NOT NULL,
    Feature_Id        INTEGER     NOT NULL,
    Impact            NUMBER      NOT NULL,
    --
    -- primary key
    --
    CONSTRAINT PK_PM_Trend_Impact PRIMARY KEY(Trigger_Id, Feature_Id),
    --
    -- foreign key(s)
    --
    CONSTRAINT FK_Trend_Impact_Trigger_Id
    FOREIGN KEY(Trigger_Id) REFERENCES PM_Trigger(Trigger_Id),
    CONSTRAINT FK_Tred_Impact_Feature_Id
    FOREIGN KEY(Feature_Id) REFERENCES PM_Feature(Feature_Id) ON DELETE CASCADE
)
TABLESPACE WM_Client;

CREATE TABLE PM_Graph(
    Id               INTEGER CONSTRAINT PK_PM_Graph PRIMARY KEY,
    FolderId         INTEGER NOT NULL
       CONSTRAINT FK_PM_Graph_FolderId REFERENCES PM_Folder(id),
    UserId           INTEGER NOT NULL
       CONSTRAINT FK_PM_Graph_IserId REFERENCES PM_User(id),
    query_id         NUMBER,
    Name             VARCHAR2(64) NOT NULL,
    Description      VARCHAR2(255),
    Changed          DATE DEFAULT TRUNC(SYSDATE, 'MI'))
TABLESPACE WM_Client;
--
-- index required to prevent duplicate documents in folder
--
CREATE UNIQUE INDEX UNQ_PM_Graph ON PM_Graph(FolderId, UPPER(Name))
TABLESPACE WM_Client;

CREATE SEQUENCE SQ_Graph
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE;

CREATE TABLE PM_GraphItem(
    Id               INTEGER CONSTRAINT PK_PM_GraphItem PRIMARY KEY,
    GraphId          INTEGER NOT NULL
       CONSTRAINT FK_PM_GraphItem_GraphId REFERENCES PM_Graph(Id) ON DELETE CASCADE,
    Ordinal          INTEGER NOT NULL,
    GraphType        VARCHAR2(32) NOT NULl,
    Name             VARCHAR2(64),
    PosX             INTEGER,
    PosY             INTEGER,
    Width            INTEGER,
    Height           INTEGER,
    Field            VARCHAR2(4000))
TABLESPACE WM_Client;

CREATE SEQUENCE SQ_GraphItem
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE;


CREATE TABLE PM_GraphItemProp(
    Id               INTEGER       NOT NULL,
    GraphItemId      INTEGER       NOT NULL, -- graph item id (parent)
    Ordinal          INTEGER       NOT NULL,
    PropName         varchar(32)   not null,
    PropValue        varchar(64)   ,
    --
    -- foreign key(s)
    --
    CONSTRAINT PM_GraphItemProp_GraphIte_fk FOREIGN KEY(GraphItemId) REFERENCES PM_GraphItem(Id) ON DELETE CASCADE,
    --
    -- primary key
    --
    CONSTRAINT PK_PM_GraphItemProp PRIMARY KEY(Id) using index tablespace wm_client
)
TABLESPACE WM_Client;
--
-- sequence for PM_GraphItemProp.Id
--
CREATE SEQUENCE SQ_GraphItemProp
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE
;

CREATE TABLE PM_DirectReport(
    Id               INTEGER CONSTRAINT PK_PM_DirectReport PRIMARY KEY,
    ReportId         INTEGER)
TABLESPACE WM_Client;

CREATE SEQUENCE SQ_DirectReport
    START WITH 1000
    INCREMENT BY 1
    NOMAXVALUE
    NOCYCLE;


--
-- configurable client code...
-- 12.05.00 George Kempf - authored
-- 12.19.00 Kevin Wagner - debugged George's code
-- 01.04.01 Kevin Wagner - added PM_SOFTALARM and PM_ALARMTYPE
-- 01.29.01 Kevin Wagner - updated to new spec.
-- 2001.02.08 Kevin Wagner - SEQUENCE SQ_ALARMTYPE, SEQUENCE SQ_SOFTALARM
-- 2001.04.03 Kevin Wagner - removed PM_SoftAlarm and PM_AlarmType

--
-- remove the pm_document because this table is nolonger needed, and we
-- need this name for enterprise 
--
--          1         2         3
-- 123456789012345678901234567890
-- [put name here to test length]
--
--PROMPT Creating table pm_document...
--CREATE TABLE pm_document (
--  id              NUMBER,
--  document_type   VARCHAR2(64),  -- client category, e.g., 'Traffic Template'
--  document_table  VARCHAR2(30),  -- table in which documents of this type are stored
--  CONSTRAINT pm_document_pk PRIMARY KEY ( id ),
--  CONSTRAINT pm_document_unq UNIQUE ( document_type )
--        );
--
----Populating PM_Document Data
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 10, 'Config Entity Report' , 'PM_Report' );
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 20, 'Config Map Report', ' PM_Report' );
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 30, 'Config Scenario Report', 'PM_Report' );
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 40, 'Config Template', 'PM_Template' );
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 50, 'CPFail Count Report', 'PM_Report' );
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 60, 'CPFail Count Template', 'PM_Template' );
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 70, 'CPFail Message Report', 'PM_Report' );
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 80, 'CPFail Message Template', 'PM_Template' );
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 90, 'External File', 'null' );
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 100, 'Folder', 'PM_Folder');
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 110, 'Graph Template', 'PM _Graph' );
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 120, 'Map Template', 'PM_Map' );
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 130, 'Report Group Schedule', 'RG_Schedule_view' );
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 140, 'Scenario', 'PM_Scenario' );
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 150, 'Schedule Group', 'Job_Group' );
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 160, 'SES Schedule', 'SES_Schedule_view' );
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 170, 'SES Study', 'SES_Study' );
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 180, 'Soft Alarm', 'SA_Softalarm' );
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 190, 'Soft Alarm Schedule' , 'SA_Schedule_view' );
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 200, 'Soft Alarm Template' , 'SA_Template' );
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 210, 'Soft Alarm View', 'PM_SAViewer' );
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 220, 'Traffic Report', 'PM_Report' );
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 230, 'Traffic Template', 'PM_Template' );
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 240, 'UDR Report', 'PM_Report' );
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 250, 'UDR Template', 'PM_Template' );
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 260, 'Channel Set Plan', 'PM_ChanSetPlan' );
--INSERT INTO pm_document ( id, document_type, document_table ) VALUES ( 270, 'Shared Folder', 'PM_Folder' );
--commit;



PROMPT Creating table pm_tools...
CREATE TABLE pm_tools (
  tool_name VARCHAR2(64),
  description VARCHAR2(128),
  position  INTEGER,
    CONSTRAINT pm_tools_pk PRIMARY KEY (tool_name) USING INDEX TABLESPACE wm_client
) TABLESPACE wm_client;

PROMPT Creating table pm_app_names...
CREATE TABLE pm_app_names (
  app_id    INTEGER,
  app_name  VARCHAR2(26),
  active    CHAR(1) DEFAULT 'Y',
    CONSTRAINT pm_app_names_pk PRIMARY KEY (app_id) USING INDEX TABLESPACE wm_client,
    CONSTRAINT pm_app_names_app_name_uq UNIQUE (app_name) USING INDEX TABLESPACE wm_client,
    CONSTRAINT pm_app_names_app_name_nn CHECK (app_name IS NOT NULL),
    CONSTRAINT pm_app_names_active_ck CHECK (active IN ('Y','N'))
) TABLESPACE wm_client;
 
PROMPT Creating table pm_tabs...
CREATE TABLE pm_tabs (
  tab_id    INTEGER,
  control_name  VARCHAR2(64),
  description VARCHAR2(64),
    CONSTRAINT pm_tabs_pk PRIMARY KEY (tab_id) USING INDEX TABLESPACE wm_client,
    CONSTRAINT pm_tabs_control_name_uq UNIQUE (control_name)  USING INDEX TABLESPACE wm_client, 
    CONSTRAINT pm_tabs_control_name_nn CHECK (control_name IS NOT NULL)
) TABLESPACE wm_client;

PROMPT Creating table pm_prop_options_tabs...
CREATE TABLE pm_prop_options_tabs (
  optiontab_id  INTEGER,
  tab_type  VARCHAR2(26),
  tab_name  VARCHAR2(26),
  tab_label VARCHAR2(26),
  ctl_prog  VARCHAR2(64),
  comments  VARCHAR2(128),
  tab_active  CHAR(1) DEFAULT 'Y',
    CONSTRAINT pm_prop_options_tabs_pk PRIMARY KEY (optiontab_id) USING INDEX TABLESPACE wm_client,
    CONSTRAINT pm_prop_options_tabs_uq UNIQUE (tab_name, tab_label, ctl_prog) USING INDEX TABLESPACE wm_client,
    CONSTRAINT pm_prop_options_ctl_prog_nn CHECK (ctl_prog IS NOT NULL),
    CONSTRAINT pm_prop_options_tab_label_nn CHECK (tab_label IS NOT NULL),
    CONSTRAINT pm_prop_options_tab_type_nn CHECK (tab_type IS NOT NULL),
    CONSTRAINT pm_prop_options_tab_name_nn CHECK (tab_name IS NOT NULL),
    CONSTRAINT pm_prop_options_tab_active_nn CHECK (tab_active IN ('Y','N'))
) TABLESPACE wm_client;

PROMPT Creating table pm_global_classes...
CREATE TABLE pm_global_classes (
  class_id  INTEGER,
  global_class  VARCHAR2(64),
  description VARCHAR2(128),
    CONSTRAINT pm_global_classes_pk PRIMARY KEY (class_id) USING INDEX TABLESPACE wm_client,
    CONSTRAINT pm_global_classes_global_cl_uq UNIQUE (global_class) USING INDEX TABLESPACE wm_client,
    CONSTRAINT pm_global_classes_global_cl_ck CHECK (global_class IS NOT NULL)
) TABLESPACE wm_client;

PROMPT Creating table pm_groups...
CREATE TABLE pm_groups(
  group_id    INTEGER,
  group_name    VARCHAR2(26),
  group_description VARCHAR2(128),
  group_active  CHAR(1) DEFAULT 'Y',
  position    INTEGER,
    CONSTRAINT pm_groups_pk PRIMARY KEY (group_id) USING INDEX TABLESPACE wm_client,
    CONSTRAINT pm_groups_group_name_uq UNIQUE (group_name) USING INDEX TABLESPACE wm_client,
    CONSTRAINT pm_groups_group_active_ck CHECK (group_active IN ('Y','N')),
    CONSTRAINT pm_groups_group_name_nn CHECK (group_name IS NOT NULL)
) TABLESPACE wm_client;

PROMPT Creating table job_type...
CREATE TABLE job_type (
  id      INTEGER,
  description   VARCHAR2(64),
        schedule_view   VARCHAR2(32),
  post_job_proc     VARCHAR2(255),
  post_task_proc    VARCHAR2(255),
    CONSTRAINT job_type_pk PRIMARY KEY (id) USING INDEX TABLESPACE wm_client,
    CONSTRAINT job_type_desc_nn CHECK (description IS NOT NULL)
) TABLESPACE wm_client;

PROMPT Creating table group_type_job_type...
CREATE TABLE group_type_job_type (
  group_id    INTEGER,
  job_type    INTEGER,
    CONSTRAINT group_type_job_type_pk PRIMARY KEY (group_id, job_type) USING INDEX TABLESPACE wm_client,
    CONSTRAINT group_job_group_id_fk FOREIGN KEY (group_id) REFERENCES pm_groups(group_id),
    CONSTRAINT group_job_job_type_fk FOREIGN KEY (job_type) REFERENCES job_type(id)
) TABLESPACE wm_client;

PROMPT Create tables pm_handlers...
CREATE TABLE pm_handlers (
  handler_id    INTEGER,
  handler_name    VARCHAR2(26),
  dbinitializer_class VARCHAR2(64),
  handler_active    CHAR(1) DEFAULT 'Y',
  parent_id   INTEGER,
  CLASS_ID    INTEGER,
    CONSTRAINT pm_handlers_pk PRIMARY KEY (handler_id) USING INDEX TABLESPACE wm_client,
    CONSTRAINT pm_handlers_handler_name_uq UNIQUE (handler_name) USING INDEX TABLESPACE wm_client,
    CONSTRAINT pm_handlers_handler_name_nn CHECK (handler_name IS NOT NULL),
    CONSTRAINT pm_handlers_handler_active_ck CHECK (handler_active IN ('Y','N')),
    CONSTRAINT pm_handlers_parent_id_fk FOREIGN KEY (parent_id) REFERENCES pm_handlers(handler_id),
    CONSTRAINT pm_handlers_class_id_nn CHECK (class_id IS NOT NULL),
    CONSTRAINT pm_handlers_class_id_fk FOREIGN KEY (class_id) REFERENCES pm_global_classes(class_id)
) TABLESPACE wm_client;

PROMPT Creating table pm_app_tabs...
CREATE TABLE pm_app_tabs (
  app_tab_id  INTEGER,
  app_id    INTEGER,
  tab_id    INTEGER,
  position    INTEGER,
  tabname   VARCHAR2(64),
  tablabel  VARCHAR2(24),
  active    CHAR(1) DEFAULT 'Y',
    CONSTRAINT pm_app_tabs_pk PRIMARY KEY (app_tab_id) USING INDEX TABLESPACE wm_client,
    CONSTRAINT pm_app_tabs_uq UNIQUE (app_id, tab_id, tabname, tablabel) USING INDEX TABLESPACE wm_client,
    CONSTRAINT pm_app_tabs_active_ck CHECK (active IN ('Y','N')),
    CONSTRAINT pm_app_tabs_tabname_nn CHECK (tabname IS NOT NULL),
    CONSTRAINT pm_app_tabs_tablabel_nn CHECK (tablabel IS NOT NULL),
    CONSTRAINT pm_app_tabs_tab_id_nn CHECK (tab_id IS NOT NULL),
    CONSTRAINT pm_app_tabs_app_id_nn CHECK (app_id IS NOT NULL),
    CONSTRAINT pm_app_tabs_app_id_fk FOREIGN KEY (app_id) REFERENCES pm_app_names(app_id),
    CONSTRAINT pm_app_tabs_tab_id_fk FOREIGN KEY (tab_id) REFERENCES pm_tabs(tab_id)
) TABLESPACE wm_client;

PROMPT Creating table pm_fdm_classes...
CREATE TABLE pm_fdm_classes (
  fdm_classes_id    INTEGER,
  class_name    VARCHAR2(64),
  handler_id    INTEGER,
        ordinal     INTEGER,
    CONSTRAINT pm_fdm_classes PRIMARY KEY(FDM_Classes_Id) USING INDEX TABLESPACE wm_client,
    CONSTRAINT pm_fdm_classes_uq UNIQUE (class_name, handler_id) USING INDEX TABLESPACE wm_client,
    CONSTRAINT pm_fdm_classes_class_name_nn CHECK (class_name IS NOT NULL),
    CONSTRAINT pm_fdm_classes_handler_id_nn CHECK (handler_id IS NOT NULL),
    CONSTRAINT pm_fdm_classes_handler_id_fk FOREIGN KEY (handler_id) REFERENCES pm_handlers(handler_id)
) TABLESPACE wm_client;

PROMPT Creating table pm_categories...
CREATE TABLE pm_categories (
  category_id   INTEGER,
  class_id    INTEGER,
  property_id   INTEGER,
  categoryfunction  VARCHAR2(64),
  categoryname    VARCHAR2(32),
  categorydocname   VARCHAR2(32),
  iconlabel   VARCHAR2(24),
  icon      INTEGER,
  documenttypes   VARCHAR2(32),
  editor      VARCHAR2(32),
  category_active   CHAR(1) DEFAULT 'Y',
    CONSTRAINT pm_categories_pk PRIMARY KEY (category_id) USING INDEX TABLESPACE wm_client,
    CONSTRAINT pm_categories_uq UNIQUE (categoryname, categoryfunction, categorydocname, documenttypes, editor) USING INDEX TABLESPACE wm_client,
    CONSTRAINT pm_categories_property_id_nn CHECK (property_id IS NOT NULL),
    CONSTRAINT pm_categories_class_id_nn CHECK (class_id IS NOT NULL),
    CONSTRAINT pm_categories_categoryfunct_nn CHECK (categoryfunction IS NOT NULL),
    CONSTRAINT pm_categories_categoryname_nn CHECK (categoryname IS NOT NULL),
    CONSTRAINT pm_categories_categorydocna_nn CHECK (categorydocname IS NOT NULL),
    CONSTRAINT pm_categories_iconlabel_nn CHECK (iconlabel IS NOT NULL),
    CONSTRAINT pm_categories_icon_nn CHECK (icon IS NOT NULL),
    CONSTRAINT pm_categories_documenttypes_nn CHECK (documenttypes IS NOT NULL),
    CONSTRAINT pm_categories_category_acti_ck CHECK (category_active IN('Y','N')),
    CONSTRAINT pm_categories_class_id_fk FOREIGN KEY (class_id) REFERENCES pm_global_classes(class_id)
) TABLESPACE wm_client;

PROMPT Creating table pm_sched_group.......
CREATE TABLE pm_sched_group (
  group_id    INTEGER,
  can_sched   CHAR(1) DEFAULT 'Y',
  can_group   CHAR(1) DEFAULT 'Y',
  title     VARCHAR2(32),
  handler_id    INTEGER,
  category_id   INTEGER,  
    CONSTRAINT pm_sch_grp_pk PRIMARY KEY (title) USING INDEX TABLESPACE wm_client,
    CONSTRAINT pm_sch_grp_group_id_fk FOREIGN KEY (group_id) REFERENCES pm_groups (group_id),
    CONSTRAINT pm_sch_grp_handler_id_fk FOREIGN KEY (handler_id) REFERENCES pm_handlers (handler_id),
    CONSTRAINT pm_sch_grp_sched_ck CHECK (can_sched IN ('Y','N')),
    CONSTRAINT pm_sch_grp_group_ck CHECK (can_group IN ('Y','N')),
    CONSTRAINT pm_sch_grp_group_id_nn CHECK (group_id IS NOT NULL),
    CONSTRAINT pm_sch_grp_cat_id_fk FOREIGN KEY (category_id) REFERENCES pm_categories (category_id),
    CONSTRAINT pm_sch_grp_cat_id_nn CHECK (category_id IS NOT NULL),
    CONSTRAINT pm_sch_grp_handler_id_nn CHECK (handler_id IS NOT NULL)
) TABLESPACE wm_client;

PROMPT Creating table pm_group_categories...
CREATE TABLE pm_group_categories (
  category_id INTEGER,
  group_id  INTEGER,
  group_cat_id  INTEGER,
    CONSTRAINT pm_group_categories_pk PRIMARY KEY (Group_Cat_Id) USING INDEX TABLESPACE wm_client,
    CONSTRAINT pm_group_categories_uq UNIQUE (category_id, group_id) USING INDEX TABLESPACE wm_client,
    CONSTRAINT pm_group_categories_group_nn CHECK (group_id IS NOT NULL),
    CONSTRAINT pm_group_categories_categor_nn CHECK (category_id IS NOT NULL),
    CONSTRAINT pm_group_categories_categor_fk FOREIGN KEY (category_id) REFERENCES pm_categories(category_id),
    CONSTRAINT pm_group_categories_group_fk FOREIGN KEY (group_id) REFERENCES pm_groups(group_id)
) TABLESPACE wm_client;

PROMPT Creating table pm_doc_handler_tabs...
CREATE TABLE pm_doc_handler_tabs (
  id    INTEGER,
  handler_id  INTEGER,
  optiontab_id  INTEGER,
    CONSTRAINT pm_doc_handler_tabs_pk PRIMARY KEY (id) USING INDEX TABLESPACE wm_client,
    CONSTRAINT pm_doc_handler_tabs_uq UNIQUE (handler_id, optiontab_id) USING INDEX TABLESPACE wm_client,
    CONSTRAINT pm_doc_handler_tabs_optiont_nn CHECK (optiontab_id IS NOT NULL),
    CONSTRAINT pm_doc_handler_tabs_handler_nn CHECK (handler_id IS NOT NULL),
    CONSTRAINT pm_doc_handler_tabs_handler_fk FOREIGN KEY (handler_id) REFERENCES pm_handlers(handler_id),
    CONSTRAINT pm_doc_handler_tabs_optiont_fk FOREIGN KEY (optiontab_id) REFERENCES pm_prop_options_tabs(optiontab_id)
) TABLESPACE wm_client;

PROMPT Creating table pm_product_info...
CREATE TABLE pm_product_info (
  property    VARCHAR2(64),
  value     VARCHAR2(64),
    CONSTRAINT pm_product_info_pk PRIMARY KEY (property), 
    CONSTRAINT pm_product_info_value_nn CHECK (value IS NOT NULL)
) TABLESPACE WM_Client;

PROMPT Creating table pm_reg_handlers...
CREATE TABLE pm_reg_handlers (
  class_id  INTEGER,
  function_name VARCHAR2(64),
  active    char(1) DEFAULT 'Y',
    CONSTRAINT pm_reg_handlers_pk PRIMARY KEY (Function_Name) using INDEX TABLESPACE wm_client,
    CONSTRAINT pm_reg_handlers_active_nn CHECK (active IN ('Y','N')),
    CONSTRAINT pm_reg_handlers_Class_id_fk FOREIGN KEY (class_id) REFERENCES pm_global_classes(class_id)
) TABLESPACE wm_client;

PROMPT Creating sequence pm_string_table_sq...
CREATE SEQUENCE PM_StringTable_SQ START WITH 1000 INCREMENT BY 1 NOMAXVALUE NOCYCLE;

PROMPT Creating table pm_stringtable...
CREATE TABLE PM_StringTable(
    Id               INTEGER,
    Ordinal          INTEGER,
      CONSTRAINT PM_StringTable_PK PRIMARY KEY(Id, Ordinal),
    StringData       VARCHAR2(4000)
) TABLESPACE WM_Client;

PROMPT Create table pm_help_file_type...
CREATE TABLE pm_help_file_type (
        id              NUMBER,
        name            VARCHAR2(64),
        id_base         NUMBER(8),
        id_range        NUMBER(6),
        has_f1_help     CHAR(1),
                CONSTRAINT pm_help_file_type_pk PRIMARY KEY (id) USING INDEX TABLESPACE wm_client,
                CONSTRAINT pm_help_file_type_has_f1_he_ck CHECK ( has_f1_help IN ( 'Y', 'N' ))
) TABLESPACE wm_client;

PROMPT Create table pm_help_file...
CREATE TABLE pm_help_file (
        id              NUMBER,
        help_file_type  NUMBER,
                CONSTRAINT pm_help_file_help_file_type_fk FOREIGN KEY (help_file_type) 
                REFERENCES pm_help_file_type( id ),
        server_path     VARCHAR2(255),
        client_folder   VARCHAR2(255),
        client_name     VARCHAR2(64),
        menu_text       VARCHAR2(40),
        menu_position   NUMBER(2),
        version         VARCHAR2(10),
        feature_license VARCHAR2(64) default NULL,
        html_folder     VARCHAR2(64), 
                CONSTRAINT pm_help_file_pk PRIMARY KEY (id) USING INDEX TABLESPACE wm_client
) TABLESPACE wm_client;

CREATE UNIQUE INDEX pm_help_file_client_name_unq ON pm_help_file( client_name );
CREATE UNIQUE INDEX pm_help_file_menu_positi_unq ON pm_help_file( menu_position );

PROMPT Create sequence pm_help_file_sq...
CREATE SEQUENCE pm_help_file_sq
  START WITH 1000
  INCREMENT BY 1
  NOMAXVALUE
  NOCYCLE;

PROMPT Create trigger pm_help_file_ins_tr
CREATE OR REPLACE TRIGGER pm_help_file_ins_tr
  BEFORE INSERT ON pm_help_file FOR EACH ROW
BEGIN
  IF ( :NEW.id IS NULL ) THEN
    SELECT pm_help_file_sq.NEXTVAL INTO :NEW.id FROM DUAL;
  END IF;
END;
/
SHOW ERRORS


CREATE OR REPLACE VIEW WM_DATAAVAIL_ENTITY_VW AS
SELECT wec.ENTITY_ID, we.ENTITY_NAME, we.DEFAULTNAMEEXPR, we.LABELEXPR,
       we.ENTITYORDEREXPR, wec.PARENT_ENTITY_ID, we.ISGROUP,
       DECODE( wcd.entity_id, NULL, 'N', 'Y' ) HAS_DA_SHOW, hier_level
FROM   WMN_CONTEXT wc,
       (select ENTITY_ID, PARENT_ENTITY_ID,context_id, level hier_level
        FROM WMN_ENTITY_CONTEXT
        WHERE  context_id=(select context_id
                           from wmn_context where
                           upper(context_name) = 'DATA AVAILABILITY')
        connect by parent_entity_id= prior entity_id
        and context_id = prior context_id
        start with parent_entity_id is NULL
        and context_id=(select context_id
                        from wmn_context where
                        upper(context_name) = 'DATA AVAILABILITY'))  wec,
       WMN_ENTITY we,
       (SELECT a.entity_id entity_id
          FROM WMD_FIELD_DEF a,
               WMD_DICT_NAME b,
               WMD_DATA_SOURCE c
         WHERE a.field_id = b.field_id
           AND a.datasource_id = c.datasource_id
           AND UPPER( c.datasource_name ) LIKE '%CLIENT%'
           AND UPPER( b.name ) = 'DA_SHOW' ) wcd
WHERE wec.CONTEXT_ID = wc.CONTEXT_ID
  AND we.ENTITY_ID = wec.ENTITY_ID
  AND we.ENTITY_ID = wcd.entity_id(+)
order by hier_level;
 

COMMIT;

CREATE OR REPLACE VIEW "JOB_GROUP_VIEW" ("ID", "FOLDERID", "USERID", "NAME", "DESCRIPTION", "CHANGED", "GROUP_TYPE", "RPTCOUNT") 
  AS 
    SELECT
          g.Id,
          g.FolderId,
          g.UserId,
          g.Name,
          g.Description,
          g.Changed,
          g.group_type,
          COUNT(i.Group_Id) RptCount
    FROM
          Job_Group g,
          Job_Group_Item i
    WHERE
          g.Id = i.Group_Id(+)
    GROUP BY
          g.Id,
          g.FolderId,
          g.UserId,
          g.Name,
          g.Description,
          g.Changed,
          g.group_type;


EXIT


-- ----------------------------------------------------------------------------
--
-- Local Variables:
-- mode: pls
-- End:
