#!/usr/bin/env perl -w
 
# -----------------------------------------------------------------------
# Encapsulate all global variables. Any use of global will need to 
# explicitly mention this package name.

package global ;

#### External variables ####

# Set DEBUG on if TRACE is defined in environment
$DEBUG = exists $ENV{TRACE} ? 1 : 0 ;
print "Trace is on\n" if $DEBUG ;

# Define default Data dictionary parameters. 
# Note:
# 1) DD_* fields have # higher precedence over the corresponding AUTO_* fields.
# 2) OUTPUT_ORDER determines what fields are output and in what order

%ddParam = (
  DD_ENTITY		=> "",		# Entity Name
  DD_FIELD_NAME		=> "",
  DD_FIELD_HEADER	=> "",
  DD_FIELD_HEADER2	=> "",
  DD_DATATYPE		=> "",
  DD_FIELD_TYPE		=> "C",
  DD_FIELD_DESC		=> "",
  DD_FIELD_AGGR		=> "S",
  DD_FIELD_EMIT_AGGR	=> "U",
  DD_DATASOURCE		=> "Traffic",
  DD_FIELD_GROUP	=> "Peg",
  DD_TABLE_NAME		=> "",  # extracted from tab def file
  DD_COL_NAME		=> "",
  DD_PRECISION		=> "",
  DD_LENGTH		=> "",
  DD_READ_ONLY		=> "Y",
  DD_NULL_OK		=> "Y",
  DD_DEFAULT_VAL	=> "",
  DD_UNIT		=> "",
  DD_TECH1		=> "",
  DD_TECH2		=> "",
  DD_TECH3		=> "",
  DD_TECH4		=> "",
  DD_TECH5		=> "",
  AUTO_FIELDNAME	=> 0,   # Generate DD_FIELD_NAME using DD_COL_NAME
  AUTO_DATATYPE		=> 1,	# Generate DD_DATATYPE using oracle data type
  CREATE_DICTIONARY	=> 1,   # Create DD until it is set to '0'
  DATA_ONLY		=> 1,   # If set to 1, process only row in col def of type data
  IGNORE_FILE		=> 0,	# Ignore this col def file
  OUTPUT_DELIMITER	=> ",",  # Delimiter for output columns
  OUTPUT_ORDER		=> ""
) ;

$DEF_OUTPUT_ORDER="DD_ENTITY, DD_FIELD_NAME, DD_FIELD_HEADER, DD_FIELD_HEADER2, DD_DATATYPE, DD_FIELD_TYPE, DD_FIELD_DESC, DD_FIELD_AGGR, DD_FIELD_EMIT_AGGR, DD_DATASOURCE, DD_FIELD_GROUP, DD_TABLE_NAME, DD_COL_NAME, DD_PRECISION, DD_LENGTH, DD_READ_ONLY, DD_NULL_OK, DD_DEFAULT_VAL, DD_UNIT, DD_TECH1, DD_TECH2, DD_TECH3, DD_TECH4, DD_TECH5";

# Check if user set $OUTPUT_ORDER in the environment
$ddParam{OUTPUT_ORDER} = $DEF_OUTPUT_ORDER ;
if ( exists $ENV{OUTPUT_ORDER} ) {
  # Get all the keys
  my $ddParamKeys = join(" ", keys %ddParam) ; 
  $ddParamKeys = " " . $ddParamKeys . " " ;

  my $envStr = $ENV{OUTPUT_ORDER} ;
  $envStr =~ s/\s//g ;
  my @envArr = split (',', $envStr) ;

  my $setEnv = 1 ;
  foreach my $key (@envArr) {
    if ( $ddParamKeys !~ /\b$key\b/ ) {
      print "Invalid environment \$OUTPUT_ORDER. Ignored. Use Default.\n";
      print "Reason: Unknown data dictionary parameter, $key\n\n";
      $setEnv = 0 ;
      last ;
    }
  }
  $ddParam{OUTPUT_ORDER} = $ENV{OUTPUT_ORDER} if $setEnv ;
}

# check if user set $OUTPUT_DELIMITER in the environment
if ( exists $ENV{OUTPUT_DELIMITER} ) {
  if ( length($ENV{OUTPUT_DELIMITER}) == 1 ) {
    $ddParam{OUTPUT_DELIMITER} = $ENV{OUTPUT_DELIMITER} ;
  }
  else {
      print "Invalid environment \$OUTPUT_DELIMITER. Ignored. Use Default.\n";
      print "Reason: \$OUTPUT_DELIMITER is limited to one character\n\n";
  }
}

# Read only fields in %ddParam. I.e, cannot be edited by users in colDef
$ddReadOnlyFields="OUTPUT_ORDER, OUTPUT_DELIMITER" ;


#### Internal Parameters ####

# Define a two dimensional array to store all data dictionary
# Each element of array stores a ref to an annoymous array
@ddOutput = () ;

# DD Parameter Header; i.e., start with this string on char of line to
# set any data dictionary parameter ;
$ddHeader="DD_PARAMETERS" ;

# DD Parameter Header Pattern; i.e.; how to find match $ddHeader in a line
$ddHeaderPattern="^#\\s*${ddHeader}\\s*" ;

# Delimiter for the data dictionary parameters.
$ddDelimiter="\\t" ;

# Field delimiter for records in col def file
$colDefFieldDelimiter="\\t" ;

# string that indicates a coldef record is of data type
$colDefDataFieldStr="data" ;

# data field of col def is the second row
$colDefDataFieldInd=1 ;

# End global package

# ------------------------------------------------------------------------
package main ;

use English ;
use strict ;

mainProg:
{ # Use block so that my variables in mainProg will not leak
  # into subroutines.

  my $progName = "buildPegs" ;
  my $tableDefFile = $ARGV[0] ;
  my $outputFile = $ARGV[1] ;

  # Checking input command-line parameters
  defined $tableDefFile or die "Usage: $progName <tableDefFile> <output file>\n" ;
  defined $outputFile   or die "Usage: $progName <tableDefFile> <output file>\n" ;

  # open for writing data dictionary
  open (OUTFILE, ">$outputFile") or die "Cannot open file for write: $!\n" ;

  # Process table def file
  my %tableColFile = &procTableDefFile($tableDefFile) ;

  # Process the column files

  my $hashRef = \%tableColFile ;
  & procColFile ( $hashRef  );

  & printDDOutput (*OUTFILE);

} #End mainProg

#-------------------------------------------------------------------------
# Sub-routines
# Note: all sub-routines are part of main package.
# ------------------------------------------------------------------------

#######################################################
# Subroutine: procTableDefFile
# Purpose: Extract database table name and the column
#          filename from each record put into a hash
# Input: Argument 1 - table file name. The file should
#                     be in the directory where this 
#                     script is executed.
# Return: A hash where key is the database table name
#         and the value is the name of the column file.
# Uses  : $global::DEBUG
#######################################################

sub procTableDefFile
{
  my %tableColFile = () ;
  my $tableDefFile = shift or return %tableColFile ;

  # Read each record into the hash, %tableColFile where the key is the database
  # table name and value is the col file name.

  open(TABLE_DEF_FILE, "<$tableDefFile") or die "Cannot open $tableDefFile: $!\n" ;

  my @tableRec = undef  ;

  while (<TABLE_DEF_FILE>) {
    chomp ;
    # Skip hash or blank lines 
    # *** Note, exp ! /COLUMN_DEF_FILE=/ should always be the last item
    #     in the if-condition as the $POSTMATCH in the following line depends
    #     on this match.

    next if /^#|^$/  or ! /COLUMN_DEF_FILE=/;

    @tableRec = split ( "\t", $_) ;
    $tableColFile{$tableRec[0]} = $POSTMATCH ;
    
  } # End While

  if ( $global::DEBUG ) {
    print "Read file $tableDefFile:\n" ;
    while ( (my $key, my $value) = each %tableColFile ) {
      printf( "DB Table = %-25s  Col File = %s\n", $key, $value) ;
    }
    print "\n\n" ;
  }

  return %tableColFile ;

} # End subroutine procTableDefFile ;

#######################################################
# Subroutine:  procColFile
# Purpose: 
# Input : Argument 1 - A hash where key is the database 
#                      table name and the value is the name 
#                      of the column file.
# Return: 
# Uses  : $global::ddHeaderPattern
#######################################################

sub procColFile
{
  my $hashRef = shift ;
  my %tableColFile = %$hashRef ;

  # Make sure the inout is good
  my @table = keys %tableColFile ;
  my $numTable = @table ;
  my @colFile = values %tableColFile ;
  my $numColFile = @colFile ;

  if ( $numTable == 0 || $numColFile == 0 || $numTable != $numColFile ) {
    print "procColFile: bad input \n" ;
    return -1 ;
  }

  # Proceiss each col file
  COLFILE_TAG: 
  while ( (my $key, my $val) = each %tableColFile ) {
    open (COL_FILE, "<$val") or die "Cannot open file $val :$!\n" ;
    printf("Reading file %s ... \n\n", $val) ;
   
    # Make a copy of default DD parameter so that each col file
    # starts with a fresh one
    my %localDDParam = %global::ddParam ;

    # Quick fix: Append "_tab" as all traffic has that
    $localDDParam{DD_TABLE_NAME} = $key . "_tab" ;

    print "Default Data Dictionary parameters: \n" ;
    & printDDParam(\%localDDParam) ;

    # Process each line in the file
    while (<COL_FILE>) {
       chomp ;
       next if /^$/ ;    # throw away blank lines
       
       # Set DD parameters
       if ( /${global::ddHeaderPattern}/ ) {
         & setDDParm($_, \%localDDParam) ;

	 # Check if user want to ignore this file
	 if ( $localDDParam{IGNORE_FILE} == 1 ) {
	   print "IGNORE_FILE is set. Skipping this file \.\.\.\n" ;
	   next COLFILE_TAG ;
	 }
       }
       elsif ( /^#/ ) {  # skip '#' line with no dd header pattern
         next ;
       }

       # Now create a data dictionary for this line
       & createDD($_, \%localDDParam) ;

    } # End while COL_FILE

    close (COL_FILE) ;

  } # End while ( my $key ..
 

} # End subroutine procColFile

#######################################################
# Subroutine: setDDParm
# Purpose: 
# Input : Argument 1 - A string while consists DD para-
#                      meters
# Return: 
# Uses  : ${global::ddHeaderPattern}, $global::DEBUG,
#	  $global::ddReadOnlyFields
# Note  : This routine assumes $ddParamRef is valid.
#         Otherwise, unknown behaviour can be expected.
#######################################################
sub setDDParm 
{
  (my $line, my $ddParamRef) = @_ ;

  return -1 if $line =~ /^$/ ;

  # strip the header
  $line =~ s/${global::ddHeaderPattern}// ;
  return -1 if $MATCH !~ /${global::ddHeaderPattern}/ ;

  # Now split the line in to array
  $line =~ s/\s*=/=/ ;
  $line =~ s/=\s*/=/ ;
  my @keyValArray = split($global::ddDelimiter, $line );

  # Get all the keys
  my $ddParamKeys = join(" ", keys %$ddParamRef) ; 
  $ddParamKeys = " " . $ddParamKeys . " " ;

  if ($global::DEBUG) {
    print "sub setDDParm::line:$line\n";
    print "sub setDDParm::keyValArray:@keyValArray\n" ;
    print "sub setDDParm::ddParamKeys:$ddParamKeys\n" ;
  }

  # set DD parameters
  foreach my $keyVal (@keyValArray) {

    my @keyValPair = split ( "=", $keyVal) ;
    $keyValPair[0] =~ s/\s//g ;

    if ( scalar (@keyValPair) == 1 ) { ## Note this is temporary fix
      $keyValPair[1] = "" ;
    }
    else {
      $keyValPair[1] =~ s/\s*"// ; # white spaces before double quotes and itself
      $keyValPair[1] =~ s/"\s*// ; # white spaces after double quotes and itself
    }

    # Make sure user set valid DD parameters
    if ( $ddParamKeys !~ /\b${keyValPair[0]}\b/ ) { 
      print "Unknown data dictionary parameter: ${keyValPair[0]}=$keyValPair[1]. Ignored.\n" ;
      next ;
    }

    # Check if the DD parameter is read-only
    if ( $global::ddReadOnlyFields =~ /${keyValPair[0]}/ ) {
      print "DD parameter is read-only or set via environment only: ${keyValPair[0]}=$keyValPair[1]. Ignored.\n" ;
      next ;
    }

    print "Setting ${keyValPair[0]}=$keyValPair[1].\n" ;
    ${%$ddParamRef}{$keyValPair[0]} = $keyValPair[1] ;
  }

  return 0 ;
} # End setDDParm ;

#######################################################
# Subroutine: createDD
# Purpose: 
# Input : 
# Return: 
# Uses  : @global::ddOutput, $global::colDefFieldDelimiter,
#         $global::colDefDataFieldStr, $global::colDefDataFieldInd
# Note  : This routine assumes $ddParamRef is valid.
#         Otherwise, unknown behaviour can be expected.
#######################################################
sub createDD
{
  (my $line, my $ddParamRef) = @_ ;

  return -1 if $line =~ /^$|^#/ ;
  
  # Don't create dictionary for this line
  return 0 unless $ddParamRef->{CREATE_DICTIONARY} ;

  ### set col name base on info from $line ###

  # Split the line into array
  my @colDefRec = split ($global::colDefFieldDelimiter, $line) ;

  # Check if DATA_ONLY is set
  return 0 if $ddParamRef->{DATA_ONLY} && 
       $colDefRec[$global::colDefDataFieldInd] !~ /${global::colDefDataFieldStr}/  ;

  $ddParamRef->{DD_COL_NAME} = $colDefRec[0] ;

  ### Do some proc for DD parameters prepend with AUTO_ ####

  # DD_FIELD_NAME has higher precedence if both are set
  my $unSetFieldName = 0;
  if ( $ddParamRef->{AUTO_FIELDNAME} && $ddParamRef->{DD_FIELD_NAME} eq "" ) {
    $ddParamRef->{DD_FIELD_NAME} = $ddParamRef->{DD_COL_NAME} ;
    $unSetFieldName = 1 ;
  }

  # DD_DATATYPE has higher precedence if both are set
  my $unSetDataType = 0;
  if ( $ddParamRef->{AUTO_DATATYPE} && $ddParamRef->{DD_DATATYPE} eq "" ) {

    my $dataType = "" ;
    $dataType="S" if $colDefRec[2] =~ /varchar|char/i  ;

    if ( $colDefRec[2] =~ /number/i ) {
      $dataType="I" ;
      $dataType="F" if $colDefRec[2] =~ /,/ ;
    }

    if ( $dataType ne "" ) {
      $ddParamRef->{DD_DATATYPE} = $dataType ;
      $unSetDataType = 1 ;
    }
  }

  ### Write the DD dictionary to global array ####

  # Get the data output order
  my $outputOrderStr = $ddParamRef->{OUTPUT_ORDER} ;
  $outputOrderStr =~ s/\s//g ;
  my @outputOrderArray = split(",", $outputOrderStr) ;

  # Put DD fields in an anonymous array
  my $newArrRef = [] ;
  for (my $i=0; $i< scalar(@outputOrderArray) ; $i++) {
    $newArrRef->[$i] = $ddParamRef->{$outputOrderArray[$i]} ;
  }

  if ($global::DEBUG) {
    my $debugDDArray = join ("|", @$newArrRef) ;
    print "sub createDD::newArrRef:$debugDDArray\n";
  }

  # Assign the array to next row of global DD array
  my $nextInd = scalar ( @global::ddOutput ) ;
  $global::ddOutput[$nextInd] = $newArrRef ;

  ### Clean up ####
  $ddParamRef->{DD_FIELD_NAME} = "" if $unSetFieldName ;
  $ddParamRef->{DD_DATATYPE}   = "" if $unSetDataType ;


} # End createDD ;

## Print external global parameters ##

#######################################################
# Subroutine: printDDParam
# Purpose: 
# Input : 
# Return: 
# Uses  : 
# Note  : 
#######################################################

sub printDDParam
{
  my $hashRef = shift ;
  my %ddParam = %$hashRef ;
  my $outputOrder = '' ;

  while ( (my $key, my $val) = each %ddParam ) {

    # print this one last
    if ( $key eq "OUTPUT_ORDER" ) {
      $outputOrder = $val ;
      next ;
    }

    printf( "%-25s = %s\n", $key, $val);
  }
  printf( "%-25s = %s\n", "OUTPUT_ORDER", $outputOrder );
  print "\n\n" ;
} # End printDDParam ;

#######################################################
# Subroutine: printDDOutput
# Purpose: 
# Input :  Argument 1: FILE pointer to output
# Return: 
# Uses  : @global::ddOutput, %global::ddParam
# Note  : 
#######################################################

sub printDDOutput
{
  my $fileHandle = shift or return -1;
  my @outputCols = $global::ddParam{OUTPUT_ORDER} ;

  print $fileHandle "#Data Dictionary Structure\n" ;
  print $fileHandle "#@outputCols\n" ;

  # Dump the data dictionary in  @global::ddOutput
  for (my $i=0; $i < scalar( @global::ddOutput) ; $i++ ) {
    my $arrayRef = $global::ddOutput[$i] ;
    my @outputDD = join ( $global::ddParam{OUTPUT_DELIMITER}, @$arrayRef );
    print $fileHandle "@outputDD\n" ;
  }

  return 0;
}
