-- -----------------------------------------------------------------------------
-- Schema DDL for JLoader key mapping component. This component provides a
-- relatively generic way to persist mapping data required for certain
-- vendor/application fullkey manipulations. This table is just a data
-- container. The semantics of the mapping are up to the specific application
-- to define (e.g. Ericsson UMTS RANOS uses this mapping for NodeB->Cell
-- information.
--
-- The cluster type/name fields are leading members of the PK and are used
-- to segment the data to make it more efficient to access. The reading
-- program basically reads everything for a cluster type/name pair in one
-- go. For instance, an Ericsson UMTS RANOS implementation might do a read
-- that looks something like:
--
-- SELECT * FROM jloader_keymap 
--  WHERE cluster_name = 'ERC.UMTS.RANOS.RNC:FRBNTXMWRNC002'
--
-- (but it would obviously be using bind variables :-)
--
-- The cluster_name is used to define a unique namespace for this vendor,
-- application and data type. The string value itself is arbitrary, but
-- should use something like the example above to ensure correct separation
-- between applications.
--
-- The cluster_name also provides a very convenient attribute that can be
-- used to hash partition the table. This makes reading the mapping info
-- a lot faster.
--
-- 
-- meaning that a node can have many parents (which is OK). But this also
-- means that the parent cols are NOT NULL...so what to do with the ROOT
-- NODE? This just points to a row that we know doesn't exist. It's a
-- compromise between efficient and a perfect data integrity implementation.
-- -----------------------------------------------------------------------------

CREATE TABLE jloader_element
(
  cluster_name                  VARCHAR2(128) NOT NULL,
  element_id                    INTEGER NOT NULL,
  element_type                  VARCHAR2(30) NOT NULL,
  element_name                  VARCHAR2(900) NOT NULL,
  last_update                   DATE,
  CONSTRAINT pk_jloader_element PRIMARY KEY (element_id)
)
PARTITION BY HASH(cluster_name)
PARTITIONS 20
TABLESPACE wm_flexpm;

CREATE UNIQUE INDEX unq_jloader_element_tn ON jloader_element(cluster_name, element_type, element_name)
TABLESPACE wm_flexpm;


-- A classic mapping table to connect elements.
CREATE TABLE jloader_element_map
(
  cluster_name                  VARCHAR2(128) NOT NULL,
  element_id                    INTEGER NOT NULL,
  parent_element_id             INTEGER NOT NULL,
  CONSTRAINT pk_jloader_element_map PRIMARY KEY (element_id, parent_element_id),
  CONSTRAINT fk_jloader_element FOREIGN KEY (element_id) REFERENCES jloader_element(element_id)
   ON DELETE CASCADE
)
PARTITION BY HASH(cluster_name)
PARTITIONS 20
TABLESPACE wm_flexpm;

-- This view is convenient for debugging and support. However, JLoader accesses the
-- element and map data using separate queries against the two tables. This minimizes
-- the amount of data it has to process.
CREATE OR REPLACE VIEW jloader_element_v AS
SELECT e.cluster_name, e.element_id, e.element_type, e.element_name, m.parent_element_id
  FROM jloader_element e,
       jloader_element_map m
 WHERE e.cluster_name = m.cluster_name(+)
   AND e.element_id = m.element_id(+);

CREATE SEQUENCE sq_jloader_element_id;
