-- ---------------------------------------------------------------------------
-- (#) ./base/db/Oracle8i/wmlib/create_time_tables.sql BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
-- (#) 
-- COPYRIGHT 2004 WatchMark Corp.
-- ---------------------------------------------------------------------------

--
--    NAME
--      tables.sql
--
--    DESCRIPTION
--      Daylight Saving Time and Locale table definitions
--
--    This was originally created by
--    07/31/2000   JKallman - Created

--
-- Table: wm_dst_rules
--
--  id               : Unique identifier for this specific rule (referenced by wm_locales)
--  start_month      : DST starting month.  Month is 1-based, e.g., 1 for January
--  start_day        : DST starting day-of-week-in-month.
--  start_dayofweek  : DST Starting day-of-week (1=Sunday, 2=Monday, etc.)
--  start_time       : DST starting time, in minutes from 00:00 in local time
--  end_month        : DST ending month.  Month is 1-based, e.g., 1 for January
--  end_day          : DST ending day-of-week-in-month
--  end_dayofweek    : DST ending day-of-week
--  end_time         : DST ending time, in minutes from 00:00 in local time
--  dst_savings      : DST amount of time saved, in minutes
--
--  Note:  DST ends is given in standard time, not DST time, so 120 means the clocks
--         are switched back from @3 DST to @2 standard time
--
--
--  Typically, this table will follow the day-of-week-in-month rules.
--  To specify an exacty day of the month, such as March 1, set start_dayofweek to 0
--  To specify the first day of the week occurring on or after an exact dy of the month,
--    the day of the week will be negative.
--  To specify the last day of the week occurring on or before an exact day of the month,
--    make the day of the week and the day of the month negative.
--
--  Examples:   First Sunday in the start month, start_day=1, start_dayofweek=1
--              Last Sunday in the start month, start_day=-1, start_dayofweek=1
--              7th day in the start month, start_day=7, start_dayofweek=0
--              First Monday on or after 5th day of start month, start_day=5, start_dayofweek=-1
--              Last Wednesday on or before 21st of start month, start_day=-21, start_dayofweek=-4
--

CREATE TABLE wm_dst_rules(
   id              INTEGER NOT NULL 
      CONSTRAINT pk_wm_dst_rules PRIMARY KEY,
   start_month     INTEGER NOT NULL
      CONSTRAINT dst_start_month_chk CHECK ( start_month BETWEEN 1 AND 12 ),
   start_day       INTEGER NOT NULL
      CONSTRAINT dt_start_day_chk CHECK (start_day != 0 AND start_day BETWEEN -31 AND 31 ),
   start_dayofweek INTEGER NOT NULL
      CONSTRAINT dst_start_dow_chk CHECK ( start_dayofweek BETWEEN -7 AND 7 ),
   start_time      INTEGER NOT NULL
      CONSTRAINT dst_start_time_chk CHECK ( start_time BETWEEN 0 AND 1440 ),
   end_month       INTEGER NOT NULL
      CONSTRAINT dst_end_month_chk CHECK ( end_month BETWEEN 1 AND 12 ),
   end_day         INTEGER NOT NULL
      CONSTRAINT dt_end_day_chk CHECK ( end_day != 0 AND end_day BETWEEN -31 AND 31 ),
   end_dayofweek   INTEGER NOT NULL
      CONSTRAINT dst_end_dow_chk CHECK ( end_dayofweek BETWEEN -7 AND 7 ),
   end_time        INTEGER NOT NULL
      CONSTRAINT dst_end_time_chk CHECK ( end_time BETWEEN 0 AND 1440 ),
   dst_savings     INTEGER NOT NULL
)
TABLESPACE wm_flexpm;

CREATE UNIQUE INDEX UNQ_dst_rules ON wm_dst_rules(
   start_month,
   start_day,
   start_dayofweek,
   start_time,
   end_month,
   end_day,
   end_dayofweek,
   end_time,
   dst_savings
)
TABLESPACE wm_flexpm;

CREATE SEQUENCE seq_wm_dst_rules
   START WITH 1000
   INCREMENT by 1
   NOMAXVALUE
   NOCYCLE;


CREATE TABLE wm_locales(
   id              INTEGER NOT NULL 
      CONSTRAINT PK_wm_locales PRIMARY KEY,
   dst_id          INTEGER,
      CONSTRAINT FK_wm_locales_dst FOREIGN KEY( dst_id ) REFERENCES wm_dst_rules( id ),
   locale_name     VARCHAR2(64) NOT NULL,
   gmt_offset      INTEGER NOT NULL,
      CONSTRAINT gmt_offset_chk CHECK (gmt_offset BETWEEN -1440 AND 1440)
)
TABLESPACE wm_flexpm;

CREATE UNIQUE INDEX UNQ_locale_name ON wm_locales( UPPER( locale_name ) ) TABLESPACE wm_flexpm;

CREATE SEQUENCE seq_wm_locales
   START WITH 1000
   INCREMENT by 1
   NOMAXVALUE
   NOCYCLE;


-- Global table used for remote TZ scheduling
-- We removed the constraints since it is temporary storage of DST/TZR info ... be careful
CREATE GLOBAL TEMPORARY table wm_dst_tmp(
   dst_id          INTEGER,
   start_month     INTEGER,
   start_day       INTEGER,
   start_dayofweek INTEGER,
   start_time      INTEGER,
   end_month       INTEGER,
   end_day         INTEGER,
   end_dayofweek   INTEGER,
   end_time        INTEGER,
   dst_savings     INTEGER NOT NULL,
   locale_id       INTEGER,
   locale_name     VARCHAR2(64) NOT NULL,
   gmt_offset      INTEGER NOT NULL
)
ON COMMIT PRESERVE ROWS;


-- This is used for the dst_id and locale_id in wm_dst_tmp
CREATE SEQUENCE seq_wm_dst_tmp
   START WITH -1
   INCREMENT by -1
   NOMINVALUE
   NOCYCLE;


EXIT;
