#!/bin/ksh 
###########################################################################
#
# Licensed Materials - Property of IBM
# 5724-T05,5724-T06
#
# (C) Copyright IBM Corp. 2010. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
###########################################################################
# (#) ./base/pm/adm/delete_instance.sh BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
# (#)  
#

SCRIPTNAME=`basename $0`

DB_CONNECT=`crypt_util.ksh -g ${DBUSER} -c`

LOCK_DIR=/var/tmp/${DBUSER}/delete_instance/lock
mkdir -p ${LOCK_DIR}
LOCK_FILE=${LOCK_DIR}/$(basename $0).lock

trap "undo_lock;" EXIT

# ---------------------
# How to use the script
# ---------------------
usage()
{
 echo 'Usage:'
  echo 'Delete Instance'
  echo '=================================='
  echo '-entityname : (Optional) The name of the entity e.g. BTS. Requires -key option, Cannot be used with -file'
  echo '-key        : (Optional) The full key name of the entity e.g. 112-6-21. Requires -entityname option, Cannot be used with -file' 
  echo '-output     : (Mandatory) Output file name where errors are reported Two files are produced, a .summary file and a .detail file.' 
  echo '-file       : (Optional)  Name of the input file with records to be processed. '
  echo '            : The file is a comma delimited file (1 instance per line) e.g. BTS,112-6-23-23'
  echo '-mode       : (Mandatory) This can be one of the following:'
  echo '            : report: File will be generated with a list of instances that can be deleted. (No delete will take place)'
  echo '            : delete: User will be prompted to confirm the action to Delete the instances.'    
  echo '-force      : (Optional) User will not be prompted to confirm the action to Delete the instances.'
  echo '            : Force option can only be used in delete mode.'
  echo '-h          : Display usage section.'
  echo ' ' 
  echo 'Example: Delete a single instance'
  echo "$SCRIPTNAME  -entityname BTS -key 112-6-21 -output /tmp/delete_instance.txt -mode delete -force"
  echo ''
  echo 'Example: Delete using a list from a given file'
  echo "$SCRIPTNAME  -output delete_instance.txt -mode report -file delete_instance.file"
  echo ''
  exit 2
}

checkforce()
{
if [[ ${force} != "1" ]]
then 
   echo ""
   echo "Are you sure you want to continue with the Delete Y/N:"
   read answer 
   
   if [ "$answer" = 'n' ] || [ "$answer" = 'N' ]
   then
      echo "Existing Script user does not want to continue with the delete"
      exit 1
   fi
   
   if [ "$answer" != 'y' ] && [ "$answer" != 'Y' ] && [ -z "$answer" ]
   then
      echo "Existing Script user must enter either Y or N"
      exit 1
   fi
 fi     
}


function runplsqlproc
{
  sqlplus -s ${DB_CONNECT} <<SQLPLUS   
  WHENEVER SQLERROR EXIT FAILURE
  set pagesize 800
  set linesize 150
  set verify off
  set heading off
  set showmode off
  set feedback off
  set serveroutput on size unlimited
  variable returnstatus number
  DECLARE
  BEGIN
     $1;  
     COMMIT;
     :returnstatus := 0;
  EXCEPTION
     WHEN OTHERS THEN
        ROLLBACK;
        :returnstatus := 1;
        IF SQLCODE IN (-20001, -20002, -20003, -20004, -20005, -20006, -20007) THEN
           DBMS_OUTPUT.PUT_LINE(SQLERRM);
        ELSE
           RAISE;
        END IF;
  END;
  /
      
  EXIT :returnstatus
SQLPLUS
}


function runplsqlspool
{
  sqlplus -s ${DB_CONNECT} >/dev/null <<SQLPLUS   
  WHENEVER SQLERROR EXIT FAILURE
  set pagesize 10000
  set linesize 150
  set verify off
  set heading off
  set showmode off
  set feedback off
  set serveroutput on size unlimited
  column message format a150 
    
  spool ${outputfile}.summary append
  select trim(message) from wmn_delete_instance_msg_log where report_type = 'S' order by line_num; 
  spool off;
  
  spool ${outputfile}.detail append
  select trim(message) from wmn_delete_instance_msg_log where report_type = 'D' order by line_num;
  spool off;
   
  TRUNCATE TABLE wmn_delete_instance_msg_log; 
   
  EXIT
SQLPLUS
}


function die {
  echo "${SCRIPTNAME} is already running."
  echo "${SCRIPTNAME} does not support concurrent executions."
  echo "Exit Script"
  exit 1
}


function do_lock {

(set -C; /usr/ucb/echo -n ${$} > ${LOCK_FILE}) > /dev/null
case $? in
  0) #echo "Aquired lock [${LOCK_FILE}]."
  ;;
  *) # Not a clean lock, check it out.
  x="$(cat ${LOCK_FILE})"
  if [ -z "${x}" ]
  then
    #echo "Empty lockfile found [${LOCK_FILE}]. Ignoring. Proceeding."
    /usr/ucb/echo -n ${$} > ${LOCK_FILE}
  else
    [ -n "$(ps -p ${x}|nawk '$1 ~ /[0-9]/ {print $1}')" ] && die
    #echo "Stale lock file found. Overwriting."
    /usr/ucb/echo -n ${$} > ${LOCK_FILE}
  fi
  ;;
esac

}

function undo_lock {  
  x="$(cat ${LOCK_FILE})"    
  if [[ "${x}" = ${$} ]]  #if other session was killed we don;t want to rm lock file
  then 
    [ -f ${LOCK_FILE} ] && rm ${LOCK_FILE}
    [ $? -eq 0 ] || exit ${FAILURE} "Failed to remove lock file [${LOCK_FILE}]."
  fi 
}

# Setup lock file    
do_lock

########################
# Parameter validation
########################
if [ $# -eq 0 ]
 then
 usage
 exit 2
fi

entityname=''
fullkey=''
mode=''
outputfile=''
filename=''
force='0'
usage=''
noerror='0'

list=''
remove=NULL


#
# Single Quotes are used for parameters which will be passed into a pl/sql procedure
#

while [ $# -gt 0 ]
do
    case $1 in
      -mode)
          shift
          mode=\'$1\'
          ;;      
      -entityname)
          shift
          entityname=\'$1\'
          ;;
      -key) 
          shift
          fullkey=\'$1\'
          ;;
      -file)
          shift
          filename=$1
          ;;
       -output)
          shift
          outputfile=$1
          ;;   
      -force)          
          force=1
          ;;           
       -h)
          shift
          usage          
          exit 2
           ;;  
       *) echo "$SCRIPTNAME: Invalid parameter $1"
          usage
          exit 2
          ;;
     esac

     if [ $# -gt 0 ]
      then
       shift
     fi
done

#################################
# Error Checking
#################################

##check entityname and filename constraint 
if [[ ${entityname} != "" 
   && ${filename} != "" ]]
then 
  echo ""
  echo "Error"
  echo "Cannot have a value for -entityname and file at the same time."
  echo ""
  usage
  exit 1
fi

if [[ ${entityname} = "" 
   && ${fullkey} != "" ]]
then 
  echo ""
  echo "Error"
  echo "Must have a value for -entityname and -key at the same time."
  echo ""
  usage
  exit 1
fi

if [[ ${entityname} != "" 
   && ${fullkey} = "" ]]
then 
  echo ""
  echo "Error"
  echo "Must have a value for -entityname and -key at the same time."
  echo ""
  usage
  exit 1
fi

##check key and filename constraint 
if [[ ${fullkey} != "" 
   && ${filename} != "" ]]
then 
  echo ""
  echo "Error"
  echo "Cannot have a value for -key and -file at the same time."
  echo ""
  usage
  exit 1
fi

##check output file has a value
if [[ ${outputfile} = "" ]]
then
  echo ""
  echo "Error"
  echo "-output must have a value."
  echo ""
  usage
  exit 1
else
  #check dir/path is valid
  DIRNAME=`dirname ${outputfile}`  
 if [[ ! -d ${DIRNAME} ]]
 then
   echo ""
   echo "Error"
   echo "-output must have a valid directory"
   echo ""
   usage
   exit 1
 fi
fi

##mode can only be report or delete
if [[ ${mode} != "'delete'" 
   && ${mode} != "'report'" ]]
then
  echo ""
  echo "Error"
  echo "-mode can only be report or delete"
  echo ""
  usage
  exit 1
fi


##-force can only be used in delete mode.
if [[ ${force} = "1" ]]
then   
   if [[ ${mode} != "'delete'" ]]
   then
     echo ""
     echo "Error"
     echo "-force can only be used in delete mode."
     echo ""
     usage
     exit 1
   fi
fi

if [[ ${filename} != "" ]]
then
   ###check if file exists
   if [[ ! -a ${filename} ]] 
   then 
     echo ""
     echo "Error"
     echo "-file does not exist"
     echo ""
     usage
     exit 1
   fi
   ###check if file size > zero
   if [[ ! -s ${filename} ]] 
   then 
     echo ""
     echo "Error"
     echo "-file has no data"
     echo ""
     usage
     exit 1
   fi   
fi
 

#################################
# Mode is report for 1 instance
#################################
if [[ ${entityname} != "" 
   && ${fullkey} != "" 
   && (${mode} = "'report'"
   || ${mode} = "'delete'") ]] 
 then    
    if [[ ${mode} = "'delete'" ]]
    then 
       checkforce
    fi   
       
    echo ""
    echo "Review instances that will be deleted in ${mode} mode for : Entity = ${entityname}, Full key =  ${fullkey}"
    echo "All instances to be deleted can be viewed in file ${outputfile}.detail"
    echo "============================================"
    # Connect to the Database and run the query
    runplsqlproc "wm_network.instance_delete("${entityname}","${fullkey}","${mode}")"   
    # Check exit code from sql process and exit accordingly
    if [ $? -ne 0 ] 
    then
       noerror=1                
    fi
    runplsqlspool
    
#################################
# Mode is report from a file
#################################
elif [[ ${filename} != "" 
     && (${mode} = "'report'"
     || ${mode} = "'delete'")  ]] 
 then
    if [[ ${mode} = "'delete'" ]]
    then 
      checkforce
    fi  
    echo ""
    echo "Review instances that will be deleted in ${mode} mode using file ${filename}"
    echo "============================================"
    basefilename=`basename ${filename}`
    tmpfilename=/tmp/${basefilename}.tmp.`date "+%Y-%m-%d-%T"`   
    dos2unix -437 ${filename} ${tmpfilename} >/dev/null 
    line_num=0
        
    while read line_by_line
    do
          line_num=$((line_num+1))
          # For each record in the file
          echo "Processing line number ${line_num} from file"
          echo "${line_by_line}" |awk '{print substr($1,1,index($1,",")-1)}'  | read entityname
          echo "${line_by_line}" |awk '{print substr($1,index($1,",")+1),"\n"}'  | read fullkey
          # Connect to the Database and run the query
          runplsqlproc "wm_network.instance_delete('"${entityname}"','"${fullkey}"',"${mode}")"           
          if [ $? -ne 0 ] 
    	  then
	     noerror=1;
	     echo ""
	     echo "Error processing line number ${line_num} for more information see detail log ${outputfile}.detail"
	     echo""
	  else
	     echo ""
	     echo "Processed Successfully line number ${line_num}"
	     echo""	   
	  fi	  
	  runplsqlspool
          echo "============================================"         
          entityname=''
          fullkey=''

    done < ${tmpfilename}
    
    rm ${tmpfilename}
    # check to see if any data in file at all
    if [ ${line_num} -eq 0 ] 
    then
       echo "File does not contain any data"    
       exit 1
    fi             
else
   usage
   exit 2
fi

# Check exit codes from sql process and exit accordingly

echo ""
echo "Summary Log Information"
echo "======================="
cat ${outputfile}.summary
echo ""
echo "======================="
echo "Detail log: ${outputfile}.detail"
echo ""
if [[ ${noerror} = "0" ]]
then 
    echo "Script Completed Successfully" 
    echo ""
    exit 0
else
    echo "Script Completed with ERRORS for more information see detail log ${outputfile}.detail"
    echo ""
    exit 1
fi   