#!/bin/ksh
###########################################################################
#
# Licensed Materials - Property of IBM
# 5724-T05,5724-T06
#
# (C) Copyright IBM Corp. 2010. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
###########################################################################
# (#) ./base/pm/adm/delete_instance_set_table_list.sh BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
# (#)  
#

SCRIPTNAME=`basename $0`

DB_CONNECT=`crypt_util.ksh -g ${DBUSER} -c`

# ---------------------
# How to use the script
# ---------------------
usage()
{
 echo 'Usage:'
  echo 'Manage Instance Tables'
  echo '=================================='
  echo '-table         : (Mandatory) Table name. Use only if mode is add or remove.'
  echo '-column        : (Mandatory) Column name. Use only if mode is add or remove.'
  echo '               : The Column must exist in the table that the user specifies with the -table option.'  
  echo '-findkey       : (Mandatory) This parameter decides what type of column has been entered. Use only if mode is add or remove. '
  echo '               : It can be one of the following:' 
  echo '               : I for Instance_id'
  echo '               : F for FullKey'
  echo '-mode          : (Mandatory) This can be one of the following:'
  echo '               : remove:  remove record from table wmn_delete_instance_tables'
  echo '               : add:     add a new record into table wmn_delete_instance_tables'
  echo '               : report:  Output Records from table wmn_delete_instance_tables'
  echo '               :          Can only be run on its own (see example below).'
  echo '-entitycolumn  : (Mandatory) Column name of the entity field in the table. Use only if findkey is F'
  echo '-entitycoltype : (Mandatory) This parameter decides what type of column has been entered. Use only if findkey is F'
  echo '               : I for Entity_id'
  echo '               : N for Entity_name'
  echo '-h             : Display usage section.'
  echo ' ' 
  echo 'Example: insert a table and column with findkey I'
  echo "$SCRIPTNAME  -table rh_nonactive -column instance_id -findkey  I -mode add"
  echo ''
  echo 'Example: remove a table and column'
  echo "$SCRIPTNAME  -table rh_nonactive -column instance_id -findkey  I -mode remove"
  echo ''
  echo 'Example: insert a table and column with findkey F'
  echo "$SCRIPTNAME  -table rh_reparent_history -column oldfullkey -findkey  F -mode add -entitycolumn entity_name -entitycoltype N"
  echo ''
  echo 'Example: report on existing records'
  echo "$SCRIPTNAME  -mode report"  
  echo ''
  exit 2
}

function runplsqlproc
{
  sqlplus -s ${DB_CONNECT} <<SQLPLUS   
  WHENEVER SQLERROR EXIT FAILURE
  set linesize 1000
  set pagesize 80
  set verify off
  set heading off
  set showmode off
  set feedback off
  set serveroutput on size unlimited
  variable returnstatus number
  DECLARE
  BEGIN
     $1;  
     COMMIT;
     :returnstatus := 0;
  EXCEPTION
     WHEN OTHERS THEN
        ROLLBACK;
        :returnstatus := 1;
        IF SQLCODE IN (-20001, -20002, -20003, -20004, -20005, -20006, -20007) THEN
           DBMS_OUTPUT.PUT_LINE(SQLERRM);
        ELSE
           RAISE;
        END IF;
  END;
  /
      
  EXIT :returnstatus
SQLPLUS
}

function runplsqlreport
{
  sqlplus -s ${DB_CONNECT} <<SQLPLUS 
  WHENEVER SQLERROR EXIT FAILURE
  set linesize 1000
  set pagesize 80
  set verify off
  set heading on
  set showmode off
  set feedback off
  set serveroutput on size unlimited
  column Findkey format a7
  column Entity_type format a11
  Select Table_name, Column_name,FindKey,Entity_Column, Entity_type from wmn_delete_instance_tables;  
  EXIT
SQLPLUS

}

########################
# Parameter validation
########################
if [ $# -eq 0 ]
 then
 usage
 exit 2
fi

tablename=''
columnname=''
findkey=''
mode=''
usage=''
entitycolumn=''
entitycoltype=''

#
# Single Quotes are used for parameters which will be passed into a pl/sql procedure
#

while [ $# -gt 0 ]
do
    case $1 in
      -mode)
          shift
          mode=\'$1\'
          ;;      
      -table)
          shift
          tablename=\'$1\'
          ;;
      -column)
          shift
          columnname=\'$1\'
          ;;
      -findkey) 
          shift
          findkey=\'$1\'
          ;;
	-entitycolumn) 
          shift
          entitycolumn=\'$1\'
          ;;  
	-entitycoltype) 
          shift
          entitycoltype=\'$1\'
          ;;  		  
      -h)
          shift
          usage          
          exit 2
           ;;  
       *) echo "$SCRIPTNAME: Invalid parameter $1"
          usage
          exit 2
          ;;
     esac

     if [ $# -gt 0 ]
      then
       shift
     fi
done

#################################
# Error Checking
# When -mode  = add or remove then tablename/columnname and findkey are all required
#################################

echo "Start Time: `date "+%y-%m-%d-%T"`"

##mode can only be report/add/remove
if [[ ${mode} != "'add'" 
   && ${mode} != "'remove'" 
   && ${mode} != "'report'" ]]
then
  echo " "
  echo "Error"
  echo "-mode can only be add/remove/report"
  usage
  exit 1
fi

##check tablename/mode constraint 
if [[ ${tablename} != "" 
   && ${mode} != "'add'" 
   && ${mode} != "'remove'" ]]
then 
  echo " "
  echo "Error"
  echo "-table option can only have a value when the mode is remove or add"
  usage
  exit 1
fi

##check columnname/mode constraint 
if [[ ${columnname} != "" 
   && ${mode} != "'add'" 
   && ${mode} != "'remove'" ]]
then 
  echo " "
  echo "Error"
  echo "-column option can only have a value when the mode is remove or add"
  usage
  exit 1
fi

##check findkey/mode constraint 
if [[ ${findkey} != "" 
   && ${mode} != "'add'" 
   && ${mode} != "'remove'" ]]
then
  echo " "
  echo "Error"
  echo "-findkey option can only have a value when the mode is remove or add"
  usage
  exit 1
fi

if [[ ${columnname} = "" && (${mode} = "'remove'" || ${mode} = "'add'") ]]    
then
  echo " "
  echo "Error"
  echo "When -mode is remove or add then -column must have a value"
  usage
  exit 1
fi

if [[ ${tablename} = ""  
   && (${mode} = "'remove'"    
   || ${mode} = "'add'") ]]
then
  echo " "
  echo "Error"
  echo "When -mode is remove or add then -table must have a value"
  usage
  exit 1
fi

if [[ ${findkey} = "" 
   && (${mode} = "'remove'"    
   || ${mode} = "'add'") ]]
then
  echo " "
  echo "Error"
  echo "When -mode is remove or add then -findkey must have a value"
  usage
  exit 1
fi

if [[ (${findkey} != "'F'"  && ${findkey} != "'I'") && (${mode} = "'remove'" || ${mode} = "'add'") ]]
then   
      echo " "
      echo "Error"
      echo "When -mode is remove or add then -findkey must have a value of F or I"
      usage
      exit 1   
fi

if [[ ${findkey} = "'F'" ]]
then
   if [[ ${entitycolumn} = "" || ${entitycoltype} = "" ]]
   then
     echo " "
     echo "Error"
     echo "When -findkey is F then -entitycolumn and -entitycoltype must have a value"
     usage
     exit 1
   fi
   if [[ ${entitycoltype} != "'I'" && ${entitycoltype} != "'N'" ]]
   then
     echo " "
     echo "Error"
     echo "-entitycoltype Can only be I or N"
     usage
     exit 1
   fi     
fi
#################################
# Mode is report 
#################################
if [ ${mode} = "'report'" ]
then    
    echo " "
    echo "Report all Records"
    echo "============================================"
    echo " "
    runplsqlreport    
############################################
# Main Call
############################################
else 
    echo " "
    echo "Manage Tables in ${mode} mode for Table ${tablename}, Column ${columnname}, Findkey ${findkey}, EntityColumn ${entitycolumn}, EntityType ${entitycoltype}"
    echo "============================================"
    # Connect to the Database and run the query
    if [[ ${findkey} = "'F'" ]]
    then
       runplsqlproc "WM_NETWORK.manage_delete_instance_tables("${mode}","${findkey}","${tablename}","${columnname}", "${entitycolumn}","${entitycoltype}")"
    else
       runplsqlproc "WM_NETWORK.manage_delete_instance_tables("${mode}","${findkey}","${tablename}","${columnname}")"
    fi

fi

# Check exit code from sql process and exit accordingly
if [ $? -ne 0 ]
then
   echo " "
   echo "Script Failed"
   echo " "  
   exit 1
else
   echo " "
   echo "Script Completed Successfully"
   echo " "
   exit 0
fi

echo "End Time: `date "+%y-%m-%d-%T"`"