#!/bin/ksh
#----------------------------------------------------------------------------
# log-archiver.sh
#
# This script is intended to be used to archive log files generated in
# different directories under FLEXPM_HOME. The actual archiving process
# is performed by the program lar (log archiver), and this script is
# only a wrapper for that program to run on FLEXPM.
#
# This script is may be executed in two modes: SET/SCHEDULE (-s) and EXECUTE
# (-e). When it is run in SET/SCHEDULE mode, it creates a crontab entry to 
# run itself in EXECUTE mode at a specified time each day.  When it is run in 
# EXEC mode, it will set the settings for lar, read a list file 
# (archiver-log.list) and execute lar based on those settings and list.
#
# USAGE: execute log-archiver -h or see the show_usage function below.
#
# NOTES:
#
# + This script assumes the following:
#
#   -- it resides in $FLEXPM_HOME/pm/adm
#   -- there is a .profile in $FLEXPM_HOME that can be run by cron and would
#      set necessary environment variables such as FLEXPM_HOME, PATH and
#      LD_LIBRARY_PATH
#   -- a list of directories and patterns (LISTFILE) that are to be processed 
#      by lar exist in the same directory as this script and is named the same
#      as this script except that it has an extension of .list instead of .sh.
#      This list file is assumed to contain lines of 3 or more fields seperated
#      by a delimiter (LISTDELIM) where the first field is the target directory
#      relative to FLEXPM_HOME/archive.  The second field is the path to search
#      in and it is relative to FLEXPM_HOME and the rest of the fields is a set
#      of patterns.  The delimiter may be changed by modifying the value of
#      LISTDELIM under the DEFAULTS section in this script.
#   -- the program lar resides in FLEXPM_HOME/pm/bin and is executable
#
# + Default LAR settings:
#
#   -- the directory where the archived files will be placed is 
#      FLEXPM_HOME/archive
#   -- log files that are NOT older than two days will remain untouched.
#   -- archive files that are older than two weeks will be deleted.
#
#   The settings for lar may be changed under the section "LAR settings" in
#   the script.
#
#----------------------------------------------------------------------------
# 13DEC2001 - nick.talon@watchmark.com
#----------------------------------------------------------------------------
#
PROGNAME=`basename $0`
PROGDIR=`dirname $0`



# Defaults
#--------------------
#
EXEC_TIME="00:00"  # Time format is "HH:MM" where HH is 00-23, MM is 00-59
LISTDELIM="	"  # Delimiter in LISTFILE
LISTFILE="$PROGDIR/$(echo $PROGNAME | sed 's/\.sh$/\.list/g')"
FLEXPM_PROFILE="${PROGDIR}/../../.profile"

# LAR settings:
#--------------------
#
LAR_PROG="pm/bin/lar" # lar program relative to FLEXPM_HOME
ARCDIR="archive"      # archive directory name, relative to FLEXPM_HOME
DAYSTOLEAVE=2         #
WEEKSTOKEEP=2         #

#----------------------------------------------------------------------------
# some funcs

show_usage()
{
   echo "Usage: $PROGNAME <options>" >&2
   echo "Where <option> ::= -h | -s [-i][-v][-w][-t <t>] | -e [-i][-v]" >&2
   echo "-h     : show options"
   echo "-s     : schedule log-archiver" >&2
   echo "-e     : execute log-archiver" >&2
   echo "-w     : overwrite existing crontab" >&2
   echo "-i     : set log-archiver to information-only execution" >&2
   echo "-v     : set log-archiver to be verbose" >&2
   echo "-t <t> : set log-archiver to be scheduled at time <t> (HH:MI)" >&2
}

error_out()
{
   ERRORMSG=${1:-"An error was encountered."}
   ERRORCD=${2:-2}
   echo "$PROGNAME: $ERRORMSG" >&2
   exit $ERRORCD
}

#----------------------------------------------------------------------------
# main

PROGMODE="NONE"
INFO_OPTS=""
OW_CRONTAB=0

DEBUG=""
#DEBUGLOG="> $PROGDIR/$(echo $PROGNAME | sed 's/\.sh$/\.debug/g')"


while getopts wheivst: OPTS
do
   case $OPTS in
   s  ) PROGMODE="SET"
        ;;
   e  ) PROGMODE="EXEC"
        ;;
   i  ) INFO_OPTS="$INFO_OPTS -i"
        ;;
   v  ) INFO_OPTS="$INFO_OPTS -v"
        ;;
   t  ) EXEC_TIME=$OPTARG
        ;;
   w  ) OW_CRONTAB=1
        ;;
   h  ) show_usage
        exit 0
        ;;
   \? ) show_usage
        exit 2
        ;;
   esac
done
shift `expr $OPTIND - 1`


# Check if a program mode was specified
#
[ "$PROGMODE" = "NONE" ] && error_out "Execute mode not specified." 2

# Check if FLEXPM_PROFILE exists
#
[ ! -r "$FLEXPM_PROFILE" ] && error_out "Can't find $FLEXPM_PROFILE" 4

if [ "$PROGMODE" = "EXEC" ]
then
   #
   # Execute the profile to get the proper environment
   # when running in EXEC mode
   #
   . $FLEXPM_PROFILE

fi

#
# Test ENVironment variables that we need
#
[ -z "$FLEXPM_HOME" ] && error_out "FLEXPM_HOME not set." 3
[ ! -d "$FLEXPM_HOME" ] \
  && error_out "FLEXPM_HOME ($FLEXPM_HOME) does not exist." 3
#### TEST
#FLEXPM_HOME="/u01/apps/WatchMark/FlexPM/5.1_5110a/test/work.tmp"
####
[ ! -x "$FLEXPM_HOME/$LAR_PROG" ] \
  && error_out "Log Archiver $LAR_PROG does not exist or is not executable." 2

#
# Check TIME format
#
echo $EXEC_TIME | \
egrep "^[0-1][0-9]:[0-5][0-9]$|^2[0-3]:[0-5][0-9]$" > /dev/null
[ $? -ne 0 ] && error_out "Error in TIME format" 3 


if [ "$PROGMODE" = "SET" ]
then

   #-----------------------------------------------------------------------
   # SET/SCHEDULE mode ( -s )
   # create a crontab entry to run this script in EXEC mode ( -e option )
   # at the time specified by EXEC_TIME
   #-----------------------------------------------------------------------
   #

   # Split Time into Hour and Minute
   #
   EXEC_HR=`echo $EXEC_TIME | cut -f1 -d":"`
   EXEC_MI=`echo $EXEC_TIME | cut -f2 -d":"`

   #
   # run crontab without any options if the following conditions
   # are true:
   #
   #   1) no crontab file exists
   #   2) a crontab file exist but is empty
   #   3) user explicitly specified to overwrite any existing
   #      crontab file ( -w )
   #
   # otherwise append the entry to the existing crontab file by 
   # running crontab -e.  Make sure crontab uses vi as the editor
   # by exporting EDITOR and VISUAL, as the editing commands implemented
   # are vi commands.
   #
   CRONTAB_OPT=""
   vi_OPEN=""
   vi_CLOSE=""

   if [ $OW_CRONTAB -eq 0 ]
   then
      CRONTAB_LIST=`crontab -l 2> /dev/null`
      if [ $? -eq 0 ]
      then
         if [ -n "$CRONTAB_LIST" ]
         then

            export EDITOR=vi
            export VISUAL=vi

            CRONTAB_OPT="-e"
            vi_OPEN="Go"
            vi_CLOSE=":wq"

         fi
      fi
   fi

   crontab $CRONTAB_OPT <<EOF > /dev/null
${vi_OPEN}${EXEC_MI} ${EXEC_HR} * * * $FLEXPM_HOME/pm/adm/$PROGNAME -e $INFO_OPTS ${DEBUGLOG}${vi_CLOSE}
EOF

elif [ "$PROGMODE" = "EXEC" ]
then

   #-----------------------------------------------------------------------
   # EXECute mode ( -e )
   # run lar according to the parameters set
   #-----------------------------------------------------------------------
   #

   # Check List FILE
   [ ! -r "$LISTFILE" ] && error_out "Can't read $LISTFILE" 5

   # Check ARCDIR
   if [ ! -d "$FLEXPM_HOME/$ARCDIR" ]
   then
      mkdir $FLEXPM_HOME/$ARCDIR
      [ $? -ne 0 ] && error_out "Cannot create $FLEXPM_HOME/$ARCDIR" 5
   fi

   #
   # read the list file (LISTFILE) which contains lines of 3 or more fields
   # delimited by LISTDELIM.  The first field (arcpath) contains a directory
   # relative to FLEXPM_HOME/ARCDIR.  This will be used as the directory to
   # place the archived files in ( option -ad in lar).  The second field 
   # (dirpath) contains a directory relative to FLEXPM_HOME. This will be 
   # passed to the lar program as a -d parameter. The rest of the fields are
   # patterns which will be passed to lar as -p parameters.  One lar call 
   # will be issued for each unique combination of arcdir and the pattern set.
   #

   prv_pattern=""
   prv_arcpath=""
   CMD_STR=""
   CMD_CTR=0

   egrep -v "^#|^$|^[ 	]|^${LISTDELIM}" $LISTFILE \
   | grep ".*${LISTDELIM}.*${LISTDELIM}.*" \
   | sort -k 3,1 -t"${LISTDELIM}" \
   | while read -r xline
     do

        pattern=`echo "${xline}${LISTDELIM}" | cut -f3- -d"${LISTDELIM}"`
        dirpath=`echo "${xline}" | cut -f2 -d"${LISTDELIM}"`
        arcpath=`echo "${xline}" | cut -f1 -d"${LISTDELIM}"`

        if [ "${arcpath}${LISTDELIM}${pattern}" != \
             "${prv_arcpath}${LISTDELIM}${prv_pattern}" ]
        then
   
           if [ $CMD_CTR -gt 0 ]
           then
 
              CMD_TAB[$CMD_CTR]=$CMD_STR
              CMD_STR=""
  
           fi

           let "CMD_CTR = CMD_CTR + 1"

           # Check arcpath
           if [ ! -d "$FLEXPM_HOME/$ARCDIR/$arcpath" ]
           then
              mkdir $FLEXPM_HOME/$ARCDIR/$arcpath
              [ $? -ne 0 ] \
                && error_out "Cannot create $FLEXPM_HOME/$ARCDIR/$arcpath" 5
           fi

           all_patterns="-p '$pattern'"
           all_patterns=`echo "$all_patterns" \
                         | sed "s/${LISTDELIM}/' -p '/g"  \
                         | sed "s/-p ''//g"`
           CMD_STR="$CMD_STR -ad '${FLEXPM_HOME}/${ARCDIR}/$arcpath'"
           CMD_STR="$CMD_STR $all_patterns"

        fi

        [ -d "$FLEXPM_HOME/$dirpath" ] && CMD_STR="$CMD_STR -d $dirpath"

        prv_pattern=$pattern
        prv_arcpath=$arcpath

     done

   CMD_TAB[$CMD_CTR]=$CMD_STR
   CMD_STR="$FLEXPM_HOME/$LAR_PROG $INFO_OPTS -bd $FLEXPM_HOME -dl $DAYSTOLEAVE -wk $WEEKSTOKEEP"
 
   # Execute lar for all options in the CMD_TAB array that have both
   # a -p and -d options.
   #
   while [ $CMD_CTR -ne 0 ]
   do
      [ -n "$(echo xyz ${CMD_TAB[$CMD_CTR]} | \
              grep ' -d ' | grep ' -p ' | grep ' -ad ')" \
      ] && eval "$CMD_STR ${CMD_TAB[$CMD_CTR]}"
      let "CMD_CTR = CMD_CTR - 1"
   done
 
fi

#----------------------------------------------------------------------------
# END

