/*
 *=============================================================================
 *                      COPYRIGHT WatchMark Inc 2004
 *=============================================================================
 * 
 * Created on Feb 23, 2004
 * 
 */
package com.watchmark.loader.framework;
import java.io.IOException;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Map;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class MessageTypeMapReader
{
  private Map _maps;
  
  public MessageTypeMapReader(Map m)
  {
    _maps = m;
  }
  
  public void readTypeMaps(String filename) throws IOException
  {
    Element top = XmlUtil.getDocumentElement(filename);
    if (top != null)
    {
      processMessageMaps(top);
    }
    else
    {
      throw new IOException("Failed to read message-type-map file: " + filename);
    }
  }
  
  private void processMessageMaps(Element top)
  {
    /*
     * top :=
     * <message-maps xmlns:HTML="http://www.w3.org/TR/REC-xml">
     *   <message-type-map>
     *     <name>Flarion</name>
     *     <message name="HISTOGRAM_USER_LOAD">
     *       <part>field1</part>
     *       <part>field2</part>
     *       <field>mcuID</field>
     *       ...
     *     </message>
     *     <message name="foo">
     *       ...
     *     </message>
     *     ...
     *     <table>
     *       <name>...</name>
     *     </table>
     *   </message-type-map>
     *   ...
     * </message-maps>
     */
    NodeList nl = top.getElementsByTagName("message-type-map");
    for (int i=0; i<nl.getLength(); i++)
    {
      // Remember that mapName is not the same as message name. The map-name
      // is used to distinguish between different sets of messages, (say Flarion
      // and 3GPP) moving them into different namespaces.
      Node c = nl.item(i);      
      String mapName = XmlUtil.getValueByName(c, "name");
      if (mapName != null && mapName.length() > 0)
      {
        MessageTypeMap typeMap = new MessageTypeMap(mapName);
        _maps.put(mapName, typeMap);
        processMessageMap(typeMap, c);
      }
    }
  }

  private void processMessageMap(MessageTypeMap map, Node top)
  {
    NodeList nl = XmlUtil.getNamedChildren(top, "message");
    for (int i=0; i<nl.getLength(); i++)
    {
      Node c = nl.item(i);
      
      // A MessageName can either be a simple String (in the case of
      // Flarion), or a name and a list of parts (a list of fields that
      // make the message unique).
      MessageName messageName = getMessageName(c);

      Message msg = map.addMessage(messageName);
      msg.addKey("tstamp"); // tstamp is always the first key (so it's Key.F0)

      // All the keys must be defined prior to any ordinary field so we can
      // load them into the Key package, prior to pushing the actual message
      // package onto the stack (keys are usually Strings, the fields in the
      // message package are ints). Need to consider what to do with floats.
      // Perhaps have two sub-packages that we roll up from?
      boolean allKeys = true;       
      NodeList fields = XmlUtil.getNamedChildren(c, "field");
      for (int j=0; j<fields.getLength(); j++)
      {
        Node f = fields.item(j);
        String type = XmlUtil.getAttributeValue(f, "type");
        if (type != null && type.equalsIgnoreCase("key"))
        {
          if (allKeys)
          {
            msg.addKey(XmlUtil.getText(f));
          }
          else
          {
            System.err.println("Error: All keys fields must come first in a " +
              "message definition: " + messageName);
          }
        }
        else
        {
          allKeys = false;
          msg.addField(XmlUtil.getText(f));
        }
      }

      // Each table definition is used to help construct LoaderControl.txt and
      // LoadAction.txt files. This is nice, as all the configuration happens
      // in one place.
      NodeList tables = XmlUtil.getNamedChildren(c, "table");
      for (int j=0; j<tables.getLength(); j++)
      {
        Node t = tables.item(j);
        String tableName = XmlUtil.getValueByName(t, "name");
        MessageTable tbl = msg.addTable(tableName);
        
        NodeList columns = XmlUtil.getNamedChildren(t, "column");
        for (int k=0; k<columns.getLength(); k++)
        {
          Node cn = columns.item(k);
          String columnName = XmlUtil.getAttributeValue(cn, "name");
          String columnType = XmlUtil.getAttributeValue(cn, "type");
          String expression = XmlUtil.getText(cn);
          
          if (columnName != null && columnName.length() > 0 &&
            expression != null && expression.length() > 0)
          {
            if (columnType == null || columnType.length() == 0)
              columnType = "0";
            tbl.addColumn(columnName, expression, columnType);
          }
        }

        // LoadAction entries for the current table.
        NodeList actions = XmlUtil.getNamedChildren(t, "action");
        for (int k=0; k<actions.getLength(); k++)
        {
          Node a = actions.item(k);
          String when = XmlUtil.getAttributeValue(a, "when");
          String type = XmlUtil.getAttributeValue(a, "type");
          String command = XmlUtil.getValueByName(a, "command");
          String arguments = XmlUtil.getValueByName(a, "arguments");
          
          if (good(when) && good(type) && good(command) && good(arguments))
          {
            // Maybe do some more validation here?
            tbl.addAction(when, type, command, arguments);
          }
        }
      }
    }
  }
  
  private boolean good(String s)
  {
    return s != null && s.length() > 0;
  }
  
  private MessageName getMessageName(Node mm)
  {
    MessageName result = null;
    
    String nm = XmlUtil.getAttributeValue(mm, "name");
    
    result = new MessageName(nm);

    // Now get any parts they want to add to the name. This makes it easy
    // to specify a name as a list of parts (say DN entries).
    NodeList fields = XmlUtil.getNamedChildren(mm, "part");
    for (int i=0; i<fields.getLength(); i++)
    {
      Node p = fields.item(i);
      String part = XmlUtil.getText(p);
      if (part != null)
      {
        result.addPart(part);
      }
    }
    return result;
  }
}