/*
 */

package com.watchmark.loader.zipstream;

import java.util.Date;

/**
 * This class is used to represent a file entry in a cpio archive
 *
 * @author	Robin Arnold
 */
public class CpioEntry implements Cloneable 
{
    int _magic;
    int _device;
    int _inode;
    int _mode;
    int _uid;
    int _gid;
    int _numberOfLinks;
    int _remoteDeviceId;
    long _modifiedTime;
    long _filesize;
    String _name;
    long _offset;	// offset of loc header
    
    public static final int MAGIC_BYTES = 6;
    public static final String MAGIC_ODC = "070707";
    public static final String MAGIC_ASCII = "070701";

    // ODC (octal decimal character?) is from "cpio -o -H odc > foo.cpio"
    public static final int ODC_HEADER_BYTES = 70;
    public static final int ODC_DEVICE_OFFSET = 0;
    public static final int ODC_NAMESIZE_OFFSET = 53; // offset *after* 6-byte magic 
    public static final int ODC_NAMESIZE_LENGTH = 6;
    public static final int ODC_FILESIZE_OFFSET = ODC_NAMESIZE_OFFSET+ODC_NAMESIZE_LENGTH;
    public static final int ODC_FILESIZE_LENGTH = 11;
    
    public static final int C_HEADER_BYTES = 104;
    public static final int C_NAMESIZE_OFFSET=88;
    public static final int C_NAMESIZE_LENGTH=8;
    public static final int C_FILESIZE_OFFSET=48;
    public static final int C_FILESIZE_LENGTH=8;

    /**
     * Creates a new cpio entry with the specified name.
     *
     * @param name the entry name
     * @exception NullPointerException if the entry name is null
     * @exception IllegalArgumentException if the entry name is longer than
     *		  0xFFFF bytes
     */
    public CpioEntry(String name) 
    {
    	if (name == null) 
      {
        throw new NullPointerException();
      }

    	if (name.length() > 0xFFFF) 
      {
        throw new IllegalArgumentException("entry name too long");
      }
      
      _name = name;
    }

    /**
     * Creates a new zip entry with fields taken from the specified
     * zip entry.
     * @param e a zip Entry object
     */
    public CpioEntry(CpioEntry e) 
    {
      _magic = e._magic;
      _device = e._device;
      _inode = e._inode;
      _mode = e._mode;
      _uid = e._uid;
      _gid = e._gid;
      _numberOfLinks = e._numberOfLinks;
      _remoteDeviceId = e._remoteDeviceId;
      _modifiedTime = e._modifiedTime;
      _filesize = e._filesize;
      _name = e._name;
      _offset = e._offset;
    }

    /**
     * Returns the name of the entry.
     * @return the name of the entry
     */
    public String getName() 
    {
    	return _name;
    }

    /**
     * Sets the modification time of the entry.
     * @param time the entry modification time in number of milliseconds
     *		   since the epoch
     * @see #getTime()
     */
    public void setModifiedTime(long time) 
    {
    	_modifiedTime = time;
    }

    /**
     * Returns the modification time of the entry, or -1 if not specified.
     * @return the modification time of the entry, or -1 if not specified
     * @see #setTime(long)
     */
    public long getModifiedTime() 
    {
    	return _modifiedTime;
    }

    /**
     * Sets the uncompressed size of the entry data.
     * @param size the uncompressed size in bytes
     * @exception IllegalArgumentException if the specified size is less
     *		  than 0 or greater than 0xFFFFFFFF bytes
     * @see #getSize()
     */
    public void setFilesize(long size) 
    {
    	if (size < 0 || size > 0xFFFFFFFFL) 
      {
  	    throw new IllegalArgumentException("invalid entry size");
    	}
      	this._filesize = size;
    }

    /**
     * Returns the uncompressed size of the entry data, or -1 if not known.
     * @return the uncompressed size of the entry data, or -1 if not known
     * @see #setSize(long)
     */
    public long getFilesize() 
    {
    	return _filesize;
    }



    /**
     * Returns true if this is a directory entry. A directory entry is
     * defined to be one whose filesize is zero.
     * @return true if this is a directory entry
     */
    public boolean isDirectory() 
    {
    	return _filesize == 0;
    }

    /**
     * Returns a string representation of the cpio entry.
     */
    public String toString() 
    {
    	return getName();
    }


    /**
     * Returns the hash code value for this entry.
     */
    public int hashCode() 
    {
      return _name.hashCode();
    }
    
    public void setUID(int uid)
    {
      _uid = uid;
    }
    
    public void setGID(int gid)
    {
      _gid = gid;
    }
    
    public void setMode(int m)
    {
      _mode = m;
    }

    /**
     * Returns a copy of this entry.
     */
    public Object clone() 
    {
      try 
      {
        CpioEntry e = (CpioEntry)super.clone();
  	    return e;
      } 
      catch (CloneNotSupportedException e) 
      {
	    // This should never happen, since we are Cloneable
	    throw new InternalError();
      }
    }
}
