/*
 *=============================================================================
 *                      COPYRIGHT WatchMark Inc 2004
 *=============================================================================
 * 
 * Created on Feb 23, 2004
 * 
 */
package com.watchmark.loader.processor.csv;
import com.watchmark.loader.framework.*;

import com.watchmark.loader.translator.TranslatorFactory;
import com.watchmark.loader.xml.model.XMLController;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.Reader;
import java.io.Writer;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import org.apache.regexp.*;

public class CsvProcessor implements Processor
{
  private String _name;
  private Map _nameMap;
  private List _columns = new ArrayList();
  private MessageTypeMap _messageTypeMap;
  private List _patterns = new ArrayList();
  private PrintWriter _writer = null;
  private int _headerRows = 0;
  private Message _message = null;

  public CsvProcessor(String name)
  {
    _name = name;
  }

  /**
   * Configure the Processor using data it finds under this element.
   * @param e
   */
  public void configure(Element e)
  {
    // This should be the "properties" element from the zipstream.xml
    // file <processor> section. In it, we should find a reference to
    // a message-map URL that we can use to load a message type map.
    Map messageMaps = new HashMap();
    String filename = ZipStreamUtil.loadMessageTypeMap(_name, e, messageMaps, 
      TranslatorFactory.getTranslators());
      
    _messageTypeMap = (MessageTypeMap)messageMaps.get(_name);
    if (_messageTypeMap == null)
    {
      System.err.println(_name + ": No matching <message-type-map> found in: " + filename);
    }
    
    String headers = XmlUtil.getValueByName(e, "header-rows");
    if (headers != null)
      _headerRows = Integer.parseInt(headers);
  }
  
  public void setWriter(PrintWriter w)
  {
    _writer = w;
  }
  
  public void addPattern(String ptn)
  {
    try
    {
      RE r = new RE(ptn);
      _patterns.add(r);
    }
    catch (RESyntaxException e)
    {
      System.err.println("Invalid pattern for processor " + _name + ": \"" + ptn
        + "\", " + e.getMessage());
    }
  }

  /**
   * The name map is a name/value pairing, built up from parsing the
   * archive name and the current filename in the archive.
   * @param m
   */
  public void setNameMap(Map m)
  {
    _nameMap = m;
  }

  /**
   * 
   * @param filename
   * @param readerName
   * @return true if this Processor should handle the given file. The name of
   * the current reader is also passed in, as some configurations may restrict
   * processing of certain files to certain readers.
   */
  public boolean accept(String filename, String readerName)
  {
    boolean result = false;

    // We can only accept work if we're correctly configured
    if (configured())
    {
      Iterator it = _patterns.iterator();
      while (!result && it.hasNext())
      {
        RE r = (RE)it.next();
        if (r.match(filename))
        {
          String matched = r.getParen(0);
        
          // Make sure if we matched, then we matched the whole thing.
          result = matched != null && matched.equals(filename);
        }
      }
    }
    
    return result;
  }
  
  /**
   * Read this input stream and process the data in it.
   * @param is
   */
  public void process(InputStream is, String entryName)
    throws IOException
  {
    BufferedReader in = new BufferedReader(new InputStreamReader(is));
    String line;
    CSV csv = new CSV();

    int lineCount = 0;
    while ((line = in.readLine()) != null)
    {
      Iterator it = csv.parse(line);
      if (lineCount < _headerRows)
      {
        // Just read column names from the first line
        if (lineCount == 0)
          readHeader(it);
      }
      else
      {
        // Put each attribute into a hashmap to associate it with its 
        // corresponding column name we got from the header.
        Map attributes = new HashMap();
        for (int i=0; i<_columns.size() && it.hasNext(); i++)
        {
          String name  = (String)_columns.get(i);
          String value = (String)it.next();
          attributes.put(name, value);
        }
      
        if (_message != null)
        {
          _message.write(_writer, attributes);
        }
      }
    }
  }
  
  public boolean configured()
  {
    return _messageTypeMap != null;
  }
  
  private void readHeader(Iterator it)
  {
    Set names = new HashSet();
    while (it.hasNext())
    {
      String name = (String)it.next();
      _columns.add(name);
      names.add(name);
    }
    
    // Now find out what message this would map to.
    MessageName mn = getMessageName(names);

    if (mn != null)
      _message = _messageTypeMap.getMessage(mn);
    else
    {
      System.err.println("Error! No <message> matches CSV header.");
    }
  }
  
  private MessageName getMessageName(Set attributes)
  {
    // Find the first <message> entry for which all its parts can be found in
    // the attributes Map parameter.
    MessageName result = null;
    Iterator it = getMessageNameIterator();
    while (result == null && it.hasNext())
    {
      result = (MessageName)it.next();
      Iterator parts = result.getParts().iterator();

      while (result != null && parts.hasNext())
      {
        String part = (String)parts.next();
        if (!attributes.contains(part))
          result = null; // mismatch, so try another one
      }
    }
    
    return result;
  }

  public Iterator getMessageNameIterator()
  {
    List result = new ArrayList();
    
    Iterator it =_messageTypeMap.getMessageIterator();
    while (it.hasNext())
    {
      Message m = (Message)it.next();
      result.add(m.getMessageName());
    }
    
    return result.iterator();
  }

}