#!/bin/ksh
###################################################################
#
# Licensed Materials - Property of IBM
# 5724-T05,5724-T06
#
# (C) Copyright IBM Corp. 2008. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
###################################################################
# Script   : wmpreparation.sh                                     #
# Function : Server Preparation script                            #
# Usage    : wmpreparation.sh [-dbcreate] [-forcesetup] [-h]      #
###################################################################

# Set the environment
. ./env_vars

# Check variables
if [[ -z $ORACLE_HOME || -z $FLEXPM_HOME ]] ; then 
   echo "Missing required variables. Please edit the env_vars file \n"; exit;
fi

####################################
### routine to execute a command 
####################################
execCommand() {
  cmd=$1
  echo "   ->  Running $cmd"
  `$cmd`
  if [[ $? != 0 ]] ; then 
    echo "Error executing command \"$cmd\" \n"; exit 1;
  fi
}

#######################################
### routine to check/create a directory 
#######################################
checkDir() {
  ckDir=$1
  if [[ ! -d $ckDir ]] ; then
    echo "Missing required directory $ckDir "
    answer='n'
    while [[ $answer != '' && $answer != 'Y' && $answer != 'y' ]] ; do
      echo "Do you want to continue? [Y/n]"
      read answer
      if [[ $answer == 'n' ]] ; then
        exit 1;
      fi
    done
    echo "Press Enter to create the directory $ckDir or Insert device for new file system"
    read device
    if [[ -z $device ]] ; then
      execCommand "mkdir -p $ckDir"
      echo "Directory $ckDir has been created"
    else
      echo "New UFS file systems for $device is going to be created"
      answer='n'
      while [[ $answer != '' && $answer != 'Y' && $answer != 'y' ]] ; do
        echo "Do you confirm? [Y/n] "
        read answer
        if [[ $answer == 'n' ]] ; then
          echo "Add the required filesystem $ckDir and run again the script"
          exit 1;
        fi
        done
        OS=`zonename`  
        if test "${OS}" != "global"
        then
          echo "No global zone : Use the zone configuration tool to create the filesystem"
          exit 1;
        fi
        execCommand "newfs -v $device"
        execCommand "fsck -y $device"
        execCommand "mount -F ufs $device $ckDir"
        echo "UFS file systems for $device has been created"
        echo "The user must add mount table entries in /etc/vfstab for each file system."
    fi
  else
    echo "Check dir $ckDir => OK"
  fi
}

#######################################
### routine to check/create a directory 
#######################################
checkGroup() {
  GROUP=`grep "^dba:" /etc/group`
  if [[ $? != 0 ]] ; then
     execCommand "/usr/sbin/groupadd -g 600 dba "
  else
     echo "Check group => OK"
  fi
}

###########################################
### routine to check/create a user account 
###########################################
checkUser() {
  user=$1; shift; comment=$1; shift; baseDir=$1; shift; uid=$1
  USER=`grep "^$user:" /etc/passwd`
  if [[ $? != 0 ]] ; then
     echo "Missing user $user"
     if [[ ! -d $baseDir ]] ; then
       execCommand "mkdir -p $baseDir"
     fi
     execCommand "/usr/sbin/useradd -c $comment -d $baseDir -s /bin/ksh -u $uid -g 600 $user"
     echo "Created user $user"
     execCommand "mkdir -p $baseDir"
     execCommand "chown -R $user:dba $baseDir" 
     echo "Created $baseDir directory"
     echo "Set user password:"
     `passwd $user`
     echo "ORACLE_HOME=$ORACLE_HOME; export ORACLE_HOME" > $baseDir/.profile
     echo "PATH=\$PATH:\$ORACLE_HOME/bin; export PATH" >> $baseDir/.profile
     echo "LD_LIBRARY_PATH=\$LD_LIBRARY_PATH:\$ORACLE_HOME/lib; export LD_LIBRARY_PATH" >> $baseDir/.profile
     return 1
  else
     echo "Check user $user => OK"
     return 0
  fi
}

###########################################
### routine to check installed package 
###########################################
checkPkg(){
  pkg=$1
  res=`pkginfo -i | sort| grep system.*$pkg`
  if [[ $res == '' ]] ; then
    echo "Missing $pkg"
    exit 1;    
  fi
}

###########################################
### routine to check required packages 
###########################################
checkSystemRequirement (){
  checkPkg "SUNWarc";   checkPkg "SUNWbtool";
  checkPkg "SUNWhea";   checkPkg "SUNWlibm";
  checkPkg "SUNWsprot"; checkPkg "SUNWtoo";
  echo "Check system requirements for Oracle => OK"
}

###########################################
### routine to configure kernel parameter
###########################################
configureKernelParam(){
  KN=`grep "WatchMark" /etc/system`
  if [[ -z $KN ]] ; then
    execCommand "./setup-oracle-db -system"
    echo "Kernel parameters have been changed: REBOOT is needed"
  else
     echo "Kernel parameters => OK"
  fi
}

##################################
### routine to create database 
##################################
createDatabase(){
  su oracle -c ". $ORACLE_BASE/.profile;. ./env_vars; ./setup-oracle-db -create"  >> wmprep.log  2>>wmprep.log
  if [[ $? != 0 ]] ; then
    echo "Error executing setup-oracle-db -create \n"; exit 1;
  fi
}

##################################
### Usage
##################################
usage(){
     echo "Usage: $0 [-dbcreate] [-forcesetup] [-h] " 
     echo "\"-dbcreate\"   run the database creation"
     echo "\"-forcesetup\" force to run all the oracle_setup steps \n" 
}

##################################
###  Main
##################################

force=""
# Check ARGS
if [[ ! -z $1  ]] ; then 
   if [[ $1 == "-dbcreate" ]] ; then 
     createDatabase
     exit 0;
   fi

   if [[ $1 == "-forcesetup" ]] ; then 
     force="TRUE"    
   else
     usage
     exit 0;
   fi
fi

# Check existance of required file system
checkDir "/u01"; checkDir "/u02"; checkDir "/u03" ;
checkDir "/u04"; checkDir "/u05"; checkDir "/u06"

# Check existance of dba group 
checkGroup

# Check existance of oracle account
checkUser "oracle" "Oracle_R-DBMS_Admin" "${ORACLE_BASE}" "10021"
if [[ $? != 0 ]] ; then
   # Oracle has been created 
   checkSystemRequirement; configureKernelParam;
   echo "Install Oracle and run again this script"
   exit 1
fi

# Check directory
if [[ ! -d /u01/apps ]] ; then
  mkdir -p /u01/apps
  if [[ $? != 0 ]] ; then
    echo "Problem on creating the /u01/apps directory"
    exit 1
  fi
fi
if [[ ! -d /u01/apps/oracle ]] ; then
   ln -s ~oracle /u01/apps/oracle
   if [[ $? != 0 ]] ; then
     echo "Problem on creating the link to oracle base directory"
     exit 1
   fi
   execCommand "chown oracle:dba /u01/apps/oracle" 
fi
echo "Check Oracle home directory => OK"

# Check existance of flexpm account
checkUser "flexpm" "Prospect_User" "${FLEXPM_HOME}" "10020"

# Configure kernel parameter
configureKernelParam

# Check Oracle
if [[ ! -d "$ORACLE_HOME/bin" || -z $(ls "$ORACLE_HOME/bin") ]] ; then
   checkSystemRequirement
   echo "Install Oracle and run again this script"
   exit 1
fi

# Setup Oracle db script -server option
printf "%s" "Running setup scripts #"
./setup-oracle-db -server >wmprep.log  2>wmprep.log
if [[ $? != 0 ]] ; then 
  echo "Error executing \"setup-oracle-db -server\" \n"; exit;
fi

AdminDir="$ORACLE_BASE/admin"
#if [[ ! -d "${AdminDir}" ]] ; then
#  echo "\n Missing required directory $ORACLE_BASE/admin \n"; exit;
#fi

# Setup Oracle db script -drives option
SID=`echo ${ORACLE_SID} | tr '[A-Z]' '[a-z]' `  
if [[ ! -z $force || ! -d "/u01/${SID}" || ! -d "${ORACLE_BASE}/admin/${SID}/create" ]] then
  printf "%s" "#"
  ./setup-oracle-db -drives  >> wmprep.log  2>>wmprep.log
  if [[ $? != 0 ]] ; then
    echo "Error executing \"setup-oracle-db -drives\" \n"; exit;
  fi
else
  printf "\n%s\n" "Skipped setup oracle db drives\" => ALREADY EXECUTED";
fi

# Check Sizing Tool results
if [[ ! -f /tmp/results.txt ]] ; then
  echo "\nThe file results.txt is missing. Copy result.txt under /tmp and run again this script \n"
  exit 1;
fi

# Setup Oracle db script -sizing option
ToolBase=`dirname $0 `
SidDirectory="${ToolBase}/${SID}"
if [[ ! -z $force  || ! -d "${SidDirectory}" ]] then
  printf "%s" "#"
  ./setup-oracle-db -sizing  >> wmprep.log 2>>wmprep.log
  if [[ $? != 0 ]] ; then
    echo "Error executing \"setup-oracle-db -sizing\" \n"; exit;
  fi
else
  printf "\n%s\n" "Skipped setup oracle db sizing\" => ALREADY EXECUTED ";
fi

# Setup Oracle db script -config option
if [[ ! -z $force  || ! -f "${AdminDir}/${SID}/create/create_DB_${SID}.sql" ]] then 
  printf "%s" "#"
  ./setup-oracle-db -config  >> wmprep.log  2>>wmprep.log
  if [[ $? != 0 ]] ; then
    echo "Error executing \"setup-oracle-db -config\" \n"; exit;
  fi
else
  printf "\n%s\n" "Skipped setup oracle db config\" => ALREADY EXECUTED ";
fi

# Database creation
answer='y'
echo "\nThe environment is ready for the database creation"
while [[ $answer != '' && $answer != 'N' && $answer != 'n' ]] ; do
  echo "Do you want to exit and check the scripts? [y/N]"
  read answer
  if [[ $answer == 'y' || $answer == 'Y' ]] ; then
    echo "Check the scripts and run \"$0 -dbcreate\" "
    exit 1;
  fi
done
createDatabase

printf "\n%s\n" "*** End ****"
exit 0;