###########################################################################
#
# Licensed Materials - Property of IBM
# 5724-T05,5724-T06
#
# (C) Copyright IBM Corp. 2007,2009. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
###########################################################################
# (#) ./base/sw/perl/lib/Utils/SSL.pm BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
# (#) 
#
###################################################################
# Module   : Utils/SSL.pm                                         #
# Function : This module allows for the ssl config, enabling      #
#            and disabling modifications to be made               #
#                                                                 #
###################################################################
package Utils::SSL;


use strict;
use Env;
use File::Copy;
use SDBM_File;
use Utils::SysVal;
use FileHandle;
use Data::Dumper;

sub new {
       my $class = shift;
       
       my $self = {
                     'LOG_FILE'               =>     join("/",$ENV{'PM_LOGS'}, "ssl.log"),
                     'SSL_SYS_VAL_NAME'       =>     'PWebSSLEnabled',
                     'SSO_SYS_VAL_NAME'       =>     'PWebSSOEnabled',
                     'SSL_HOME_DIR'           =>     join ("/",$ENV{'FLEXPM_HOME'},"SSL"),
                     'LOCALHOST_KEYSTORE'     =>     "localhost.keystore",
                     'KEYSTORE_PASSWORD'      =>     "keystore.password",
                     'SSL_PASSWORD_SERVICE'   =>     "ssl-password-service.xml",
                     'SALT'                   =>     "prospectisthebest",
                     'ITERATION_COUNT'        =>     17,
                     'OLD_SSL_MAP_FILE'       =>     join("/",$ENV{'FLEXPM_HOME'},"vendor",$ENV{'VENDOR'},".ssl_map"),
                     'ABORT_CODE'             =>     1
                  };
       
       bless $self, $class;
       return $self;
}

sub abortCode
{
    my $self = shift;
    if (@_) { $self->{'ABORT_CODE'} = shift }
    return $self->{'ABORT_CODE'};
}

sub oldSSLMapFile
{
    my $self = shift;
    if (@_) { $self->{'OLD_SSL_MAP_FILE'} = shift }
    return $self->{'OLD_SSL_MAP_FILE'};
}

sub salt
{
    my $self = shift;
    if (@_) { $self->{'SALT'} = shift }
    return $self->{'SALT'};
}

sub iterationCount
{
    my $self = shift;
    if (@_) { $self->{'ITERATION_COUNT'} = shift }
    return $self->{'ITERATION_COUNT'};
}

sub sslSysValName 
{
    my $self = shift;
    if (@_) { $self->{'SSL_SYS_VAL_NAME'} = shift }
    return $self->{'SSL_SYS_VAL_NAME'};
}



sub ssoSysValName 
{
    my $self = shift;
    if (@_) { $self->{'SSO_SYS_VAL_NAME'} = shift }
    return $self->{'SSO_SYS_VAL_NAME'};
}


sub logFile 
{
    my $self = shift;
    if (@_) { $self->{'LOG_FILE'} = shift }
    return $self->{'LOG_FILE'};
}

sub location 
{
    my $self = shift;
    if (@_) { $self->{'LOCATION'} = shift }
    return $self->{'LOCATION'};
}

sub name
{
    my $self = shift;
    if (@_) { $self->{'NAME'} = shift }
    return $self->{'NAME'};
}

sub password
{
    my $self = shift;
    if (@_) { $self->{'PASSWORD'} = shift }
    return $self->{'PASSWORD'};
}

sub sslHome
{
    my $self = shift;
    if (@_) { $self->{'SSL_HOME_DIR'} = shift }
    return $self->{'SSL_HOME_DIR'};
}

sub sslPasswordService
{
    my $self = shift;
    if (@_) { $self->{'SSL_PASSWORD_SERVICE'} = shift }
    return $self->{'SSL_PASSWORD_SERVICE'};
}

sub localhostKeystore
{
    my $self = shift;
    if (@_) { $self->{'LOCALHOST_KEYSTORE'} = shift }
    return $self->{'LOCALHOST_KEYSTORE'};
}

sub keystorePassword
{
    my $self = shift;
    if (@_) { $self->{'KEYSTORE_PASSWORD'} = shift }
    return $self->{'KEYSTORE_PASSWORD'};
}


sub checkSSLEnabled
{
    my $self = shift;
    my $sys_val = new Utils::SysVal($self->sslSysValName());
    return ($sys_val->getVal() eq 'Y');
}


sub checkSSOEnabled
{
    my $self = shift;
    my $sys_val = new Utils::SysVal($self->ssoSysValName());
    return ($sys_val->getVal() eq 'Y');
}

sub checkSSOSet
{
    my $self = shift;
    my $sys_val = new Utils::SysVal($self->ssoSysValName());
    return (defined($sys_val->getVal()));
}

sub getSSLFileName
{
  my $self = shift;
  my ($filename) = @_;
  return join ("/", $self->sslHome, $filename);
}

sub checkSSLConfigured
{
  my $self = shift;
  if (-e $self->getSSLFileName($self->localhostKeystore()) 
      && -e $self->getSSLFileName($self->keystorePassword())
      && -e $self->getSSLFileName($self->sslPasswordService()))
  {
    return 1;
  }
  else
  { 
    return 0;
  }
}

sub configureSSL
{
  my $self = shift;
  
  my ($location, $name, $password) = @_;
  
  my $ssl_home_dir = $self->sslHome();
  my $localhost_keystore = $self->getSSLFileName($self->localhostKeystore());
  my $keystore_password = $self->getSSLFileName($self->keystorePassword());
  my $ssl_security_service = $self->getSSLFileName($self->sslPasswordService());
  my $salt = $self->salt();
  my $iteration_count = $self->iterationCount();
  
  

  # this is the security service which is used by the SSL connector above
  my $ssl_password_mbean = <<EXML;
      <server>
        <!-- Service to allow password descryption on ssl keystore password file -->
        <mbean code="org.jboss.security.plugins.JaasSecurityDomain"
           name="jboss.security:service=PBESecurityDomain">
           <constructor>
              <arg type="java.lang.String" value="encrypt-keystore-password"></arg>
           </constructor>
           <attribute name="KeyStoreURL">resource:localhost.keystore</attribute>
           <attribute name="KeyStorePass">{CLASS}org.jboss.security.plugins.FilePassword:\${jboss.server.home.dir}/conf/keystore.password</attribute>
           <attribute name="Salt">$salt</attribute>
           <attribute name="IterationCount">$iteration_count</attribute>
        </mbean>
      </server>
EXML
  
  $self->_printStart();
  
  # get & validate params - either from options passed in or from stdin
  (defined($location) ? $self->location($location) : $self->_queryLocation());
  $self->_validateLocation();  
  (defined($name) ? $self->name($name) : $self->_queryName());
  $self->_validateName();  
  (defined($password) ? $self->password($password) : $self->_queryPassword());
  $self->_validatePassword();  
  
  # now we have everything validated we can create the keystore password file, jboss service and copy everything to the SSL dir
  
  
  # create the SSL (if necessary) directory and clear out any existing config in there
  
  if (-d $ssl_home_dir)
  {
    unlink($localhost_keystore) if (-e $localhost_keystore);
    unlink($keystore_password) if (-e $keystore_password);
    unlink($ssl_security_service) if (-e $ssl_security_service);    
  }
  else
  { 
    mkdir $ssl_home_dir;
  }
  
  
  # copy over the keystore
  my $keystore = join("/",$self->location(),$self->name());
  copy ($keystore,$localhost_keystore) or $self->_abort("Could not copy keystore");
  
  # now generate the encrypted password file
  my $password_cp = $ENV{'FLEXPM_HOME'}."/pm/pas/".$ENV{'JBOSS_VERSION'}."/server/default/lib/jbosssx.jar";
  my $passwd = $self->password();
  my $password_command =  join(" ","java","-cp",$password_cp,"org.jboss.security.plugins.FilePassword",$self->salt() ,$self->iterationCount(), $self->password(), $keystore_password);
  
  system($password_command);
  
  # now generate the jboss ssl service xml file
  
  my $fh = FileHandle->new($ssl_security_service, "w") or $self->_abort("Error obtaining filehandle for $ssl_security_service: " . $!);
  $fh->print($ssl_password_mbean);
  $fh->close() or $self->_abort("Error closing filehandle for $ssl_security_service: " . $!);
  
  if ($self->checkSSLConfigured())
  {
    $self->_printLog("SSL Configured Successfully");
  }
  else
  {
    $self->_abort("Error configurating SSL");
  }
  
  $self->_printEnd();
  
}

sub enableSSL
{
  my $self = shift;
  
  $self->_printStart();
  $self->_printLog("Enabling SSL");
  
  $self->_abort("SSL not configured") unless ($self->checkSSLConfigured());
  
  $self->_abort("SSO Enabled cannot enable SSL") if($self->checkSSOEnabled());
  
  my $sys_val = new Utils::SysVal($self->sslSysValName());
  
  if($sys_val->getVal() eq 'Y')
  {
    $self->_abort("SSL Already Enabled");
  }
  else
  {
    $sys_val->setVal('Y');
    $self->_printLog("SSL Enabled. You must restart all middleware for the new setting to take effect.");
  }
  $self->_printEnd();
    
  
}


sub disableSSL
{
  my $self = shift;
  
  $self->_printStart();
  $self->_printLog("Disabling SSL");
  
  $self->_abort("SSL not configured") unless ($self->checkSSLConfigured());  
  
  my $sys_val = new Utils::SysVal($self->sslSysValName());
  
  if($sys_val->getVal() eq 'N')
  {
    $self->_abort("SSL Already Disabled");
  }
  else
  {
    $sys_val->setVal('N');
    $self->_printLog("SSL Disabled. You must restart all middleware for the new setting to take effect.");
  }
  $self->_printEnd();  

}

# this is used to migrate from the old motorola set-up to the new core set-up - to be run as part of the pre-upgrade steps.
sub preMigrateSSL
{
  my $self = shift;
  # we set this to 0 here as we do not want install to fail is this migration cannot succeed for some reason
  $self->abortCode(0);

  $self->_printStart();
  $self->_printLog("Pre upgrade SSL Migrate");
  
  my %old_ssl_hash;
  my $old_ssl_map_file = $self->oldSSLMapFile();
  my $old_ssl_map_file_pag = "$old_ssl_map_file.pag";
  my $old_ssl_map_file_dir = "$old_ssl_map_file.dir";
  my $ssl_enabled_file = $self->getSSLFileName(".ssl_enabled_flag");
  
  
  
  # check to see if old ssl map file exists
  if (-e $old_ssl_map_file_pag && -e $old_ssl_map_file_dir )
  {
    # now get info from old SSL map files
    $self->_printLog("SSL map files found on old system: $old_ssl_map_file_pag and $old_ssl_map_file_dir");
    
    # the preUpgradeScript will have copied the old keystore (if it existed) to $FLEXPM_HOME, so location is $ENV{'FLEXPM_HOME'}
    my $location = $ENV{'FLEXPM_HOME'};
    my $name = "keystore";
    my $keystore = join("/",$location,$name);
    # check to make sure the keystore is present
    $self->_abort("Unable to retrieve old keystore") unless (-e $keystore); 

    tie (%old_ssl_hash,'SDBM_File',"$old_ssl_map_file", 'O_RDWR|O_CREAT|O_EXCEL','0600') || $self->abort("Couldn't tie SDBM file $old_ssl_map_file: $!;");
    
    # retrieve password and enabled flag from old ssl map file
    my $password = $old_ssl_hash{'keystore'};
    my $ssl_enabled = $old_ssl_hash{'WM_USE_SSL'};
    
    #untie the file
    untie(%old_ssl_hash);

    # make sure we actually got something back before trying to configure it
    $self->_abort("Unable to retrieve information from SSL map file") unless (defined($password) && defined($ssl_enabled));    


    # update the jboss cersion as this changes from R23 to 8.0.7
    $ENV{'JBOSS_VERSION'} = "jboss-4.0.5.GA";

    # now we have configure SSL in the new system
    $self->configureSSL($location, $name, $password);
    
    # at this point just create a new file in the newly created $FLEXPM_HOME/ssl directory, this can then be used in the post install step
    my $ssl_enabled_fh = FileHandle->new($ssl_enabled_file, "w") or $self->_abort("Cannot open $ssl_enabled_file file for writing");
    $ssl_enabled_fh->print($ssl_enabled);
    $ssl_enabled_fh->close();
    
    
    # finally delete the old map files and keystore
    unlink($old_ssl_map_file_pag, $old_ssl_map_file_dir,$keystore);
  }
  else
  {
    $self->_printLog("SSL map files not found on old system - skipping SSL migration step.");
  }
  
  $self->_printEnd(); 
}  


# this is used to migrate from the old motorola set-up to the new core set-up - to be run as part of the post-install steps.
sub postMigrateSSL
{
  my $self = shift;
  # we set this to 0 here as we do not want install to fail is this migration cannot succeed for some reason
  $self->abortCode(0);

  $self->_printStart();
  $self->_printLog("Post Upgrade SSL Migrate");
  
  my $ssl_enabled_file = $self->getSSLFileName(".ssl_enabled_flag");
  
    
  # check to see if ssl flag file exists
  if (-e $ssl_enabled_file )
  {

    # read the file create in the pre upgrade step and write the flag value to the DB
    my $ssl_enabled_fh = FileHandle->new($ssl_enabled_file, "r") or $self->_abort("Cannot open $ssl_enabled_file file for read");
    my $ssl_enabled = $ssl_enabled_fh->getline();
    chomp($ssl_enabled);
    $ssl_enabled_fh->close();
    $self->_printLog("Setting ". $self->sslSysValName(). " to " . $ssl_enabled);
    # set the sys var accordingly 
    my $sys_val = new Utils::SysVal($self->sslSysValName());
    $sys_val->setVal($ssl_enabled);
    # finally delete the flag file
    unlink($ssl_enabled_file);
  }
  else
  {
    $self->_printLog("SSL flag file not found on system - skipping SSL migration step.");
  }
  
  $self->_printEnd(); 
}  

# private methods start here


sub _abort
{
  my $self = shift;
  my ($message) = @_;
  $self->_printLog($message);
  $self->_printLog("Aborting...");
  $self->_printEnd();
  # we will exit with code 1 unless this is overridden
  exit($self->abortCode());
}
  

sub _printLog {
###################################################################
#                                                                 #
###################################################################

   my $self = shift;
   my ($print_string) = @_;
   # if it ends in a tab then leave it alone otherwise newline it
   $print_string.= "\n" unless ($print_string =~ /\t$/);
   my $log_file = $self->logFile;
   open LOGFILE, ">>$log_file" or die "Unable to open $log_file for write : $!";
   print LOGFILE "$print_string";
   close LOGFILE or die "Unable to close $log_file : $!";
   print STDERR "$print_string";

}



sub _printStart {
###################################################################
#                                                                 #
###################################################################

   my $self = shift;

   my $date = `date`;
   chomp($date);

   $self->_printLog("===========================================================");
   $self->_printLog("Start : $date");

}

sub _printEnd {
###################################################################
#                                                                 #
###################################################################

   my $self = shift;

   my $date = `date`;
   chomp($date);

   $self->_printLog("End: $date");
   $self->_printLog("===========================================================");

}

sub _queryLocation
{
  my $self = shift;
  my $location;
  unless(defined($self->location))
  {
    $self->_printLog("Enter the directory location of your keystore     :\t");
    chop($location=<STDIN>);
    $location = `echo $location`;
    chomp $location;
    $self->_printLog("Keystore directory entered is : $location");
    $self->location($location);
  }
}

sub _queryName
{
  my $self = shift;
  my $name;
  unless(defined($self->name))
  {
    $self->_printLog("Enter the name of your keystore     :\t");
    chop($name=<STDIN>);
    $self->_printLog("Keystore name entered is : $name");
    $self->name($name);
  }
}  


sub _queryPassword 
{

    my $self = shift;

    my ($password,$reenter_password);

    unless(defined($self->password()))
    {
      $self->_printLog("Note: Passwords will not be displayed");
      $self->_printLog("Enter the password used to generate keystore     :\t");
      system("stty -echo");
      chop($password=<STDIN>);
      system("stty echo");
      $self->_printLog("");
      $self->_printLog("Renter the password used to generate keystore    :\t");
      system("stty -echo");
      chop($reenter_password=<STDIN>);
      system("stty echo");
      $self->_printLog("");

      if ( $password ne $reenter_password ) 
      {
          $self->_abort("Entered keystore passwords differ.");
      }

      $self->_abort("Empty password is not allowed.") if ( $password eq "" );

      $self->password($password);
    }
}

sub _validateLocation
{
  my $self = shift;
  $self->_abort("Location is invalid") unless (-d $self->location());
}

sub _validateName
{
  my $self = shift;
  my $keystore = join("/",$self->location(),$self->name());
  $self->_abort("Name does not exist") unless (-e $keystore);
}

sub _validatePassword
{
  my $self = shift;
  my $keystore = join("/",$self->location(),$self->name());
  my $password = $self->password();
  my $keytool_cmd = "keytool -list -keystore $keystore -storepass $password";
  my $ret_val = `$keytool_cmd`;
  
  $self->_abort($1) if ($ret_val =~ /Exception: (.*)$/);
}

1; # to allow module compilation