###########################################################################
#
# Licensed Materials - Property of IBM
# 5724-T05,5724-T06
#
# (C) Copyright IBM Corp. 2009. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
###########################################################################
# (#) ./base/sw/perl/lib/Utils/StartUpEnv.pm BL_core_schema_8.0.7_8.0.7.5.02_20100605  072527
# (#) 
#
###################################################################
# Module   : Utils/StartUpEnv.pm                                  #
# Function : This module allows for the environment setup at      #
#            start up time to be set                              #
#                                                                 #
###################################################################
package Utils::StartUpEnv;


use strict;
use Env;
use File::Copy;
use Utils::SysVal;
use Utils::SSL;
use Time::Local;


sub new {
       my $class = shift;
       
       my $flexpm_home       = $ENV{'FLEXPM_HOME'} || die "\$FLEXPM_HOME is not set .... aborting ";
       my $jboss_version     = $ENV{'JBOSS_VERSION'};
       my $log_file = join("/",$ENV{'PM_LOGS'}, "ssl.log");
       
       
       my $self = {
                     'SSL_OBJ'            =>     new Utils::SSL(),
                     'SSL_SYS_VAL_NAME'           =>     'PWebSSLEnabled',
                     'SSO_SYS_VAL_NAME'           =>     'PWebSSOEnabled',
                     'SSO_CONFIGURED_OK'        => 1
                  };
       
       bless $self, $class;
       return $self;
}

sub sslSysValName 
{
    my $self = shift;
    if (@_) { $self->{'SSL_SYS_VAL_NAME'} = shift }
    return $self->{'SSL_SYS_VAL_NAME'};
}

sub ssoSysValName 
{
    my $self = shift;
    if (@_) { $self->{'SSO_SYS_VAL_NAME'} = shift }
    return $self->{'SSO_SYS_VAL_NAME'};
}

sub ssoConfiguredOK 
{
    my $self = shift;
    if (@_) { $self->{'SSO_CONFIGURED_OK'} = shift }
    return $self->{'SSO_CONFIGURED_OK'};
}


sub sslObj 
{
    my $self = shift;
    if (@_) { $self->{'SSL_OBJ'} = shift }
    return $self->{'SSL_OBJ'};
}

sub checkSSLEnabled
{
    my $self = shift;
    return $self->sslObj->checkSSLEnabled();
}


sub checkSSOEnabled
{
    my $self = shift;
    return $self->sslObj->checkSSOEnabled();
}

sub checkSSOSet
{
    my $self = shift;
    my $sys_val = new Utils::SysVal($self->ssoSysValName());
    return (defined($sys_val->getVal()));
}



# used by update jboss ports to set the environment variables so the templates can be modified accordingly,
# this sets up the env as if neither SSO nor SSL is enables using a standard http connector, SSLAuthenticator

sub setStartupEnvironment
{
  my $self = shift;
  my ($jetty_port, $secure_port, $jboss_base) = @_;
  
  # standard http connector
  my $http_connector_xml = <<EXML;
                 <!-- A HTTP/1.1 Connector Configuration For Tomcat 5.5 -->
                 <Connector port="$jetty_port"
                    address="\${jboss.bind.address}"
                    maxThreads="250"
                    strategy="ms"
                    maxHttpHeaderSize="8192"
                    emptySessionPath="true"
                    enableLookups="false"
                    redirectPort="$secure_port"
                    acceptCount="100"
                    connectionTimeout="20000"
                    disableUploadTimeout="true"/>
EXML

  # check to make sure that ssl service is not in the deploy directory as we do not want to deploy this unless SSL is enabled
  my $deployed_ssl_service = join ("/" , $jboss_base, "server/default/deploy", $self->sslObj()->sslPasswordService());
  unlink ($deployed_ssl_service) if (-e $deployed_ssl_service);
  
  # set the environment accordingly
  $self->log("SSL/SSO Not Enabled, using http connector");
  $ENV{'TOMCAT_CONNECTOR_XML'}= $http_connector_xml;

  # override these
  $ENV{'CLIENT_CERT_AUTHENTICATOR'} = "org.apache.catalina.authenticator.SSLAuthenticator";
  
  if(exists($ENV{'RBAC_LOGIN_MODULE'}))
  {
    $ENV{'JAAS_LOGIN_MODULE'} = $ENV{'RBAC_LOGIN_MODULE'};
  }
  else
  {
    $ENV{'JAAS_LOGIN_MODULE'} = "com.watchmark.prospect.enterprise.api.impl.WmLdapLoginModule";  
  }



}


# used by update jboss ports to set the environment variables so the templates can be modified accordingly,
# this sets up the env when SSL is enabled using https connector, SSLAuthenticator, JaasSecurityDomain security service


sub setStartupEnvironmentSSL
{
  my $self = shift;
  my ($jetty_port, $secure_port, $jboss_base) = @_;
  
  my $salt = $self->sslObj()->salt();
  my $iteration_count = $self->sslObj()->iterationCount();

  # ssl connector that uses a security service for keystore password
  my $ssl_connector_xml = <<EXML;
      <!-- SSL/TLS Connector with encrypted keystore password configuration  -->
      <Connector  port="$secure_port"
                  address="\${jboss.bind.address}"
                  maxThreads="100"
                  minSpareThreads="5"
                  maxSpareThreads="15"
                  strategy="ms"
                  maxHttpHeaderSize="8192"
                  emptySessionPath="true"
                  scheme="https"
                  secure="true"
                  clientAuth="false"
                  sslProtocol = "TLS"
                  securityDomain="java:/jaas/encrypt-keystore-password"
                  SSLImplementation="org.jboss.net.ssl.JBossImplementation" ></Connector>
EXML




  # this will be added to the jboss-service.xml for Tomcat to ensure we have the abaility to decrypt the password
  # prior to its connector  starting up
  my $tomcat_ssl_password_dep = <<EXML;
    <!-- Add in dependancy for ssl keystore password decrpyiton service -->
    <depends>jboss.security:service=PBESecurityDomain</depends>
EXML


  # this is an addtional flag that will remain true of the ssl keystore and encrypted password files are copied over correctly
  my $ssl_config = 1;
  my $ssl = $self->sslObj();

  my $conf_dir = join ("/", $jboss_base, "server/default/conf");
  my $deploy_dir = join("/", $jboss_base, "server/default/deploy");
  my $keystore = $ssl->getSSLFileName($ssl->localhostKeystore());
  my $password = $ssl->getSSLFileName($ssl->keystorePassword()); 
  my $ssl_service = $ssl->getSSLFileName($ssl->sslPasswordService());

  # first, copy the config in to the conf dir in jboss
  unless (copy ($keystore, $conf_dir))
  {
      $self->log("WARNING: keystore file not copied from location [$keystore] to [$conf_dir]");
      $ssl_config = 0;
  }
  
  unless (copy ($password, $conf_dir))
  {
        $self->log("WARNING: password file not copied from location [$password] to [$conf_dir]");
        $ssl_config = 0;
  }
  
  unless (copy ($ssl_service, $deploy_dir))
  {
          $self->log("WARNING: ssl security service file not copied from location [$ssl_service] to [$deploy_dir]");
          $ssl_config = 0;
  }
  
  
  # indicates that copy has been successful and ssl is enabled and configrued correctly
  if ($ssl_config)
  {
    $self->log("SSL Enabled and conifigured correctly, using https connector");
    $ENV{'TOMCAT_CONNECTOR_XML'}= $ssl_connector_xml;
    $ENV{'TOMCAT_SSL_PASSWORD_DEP'} = $tomcat_ssl_password_dep;

    # override these
    $ENV{'CLIENT_CERT_AUTHENTICATOR'} = "org.apache.catalina.authenticator.SSLAuthenticator";
    $ENV{'JAAS_LOGIN_MODULE'} = "com.watchmark.prospect.enterprise.api.impl.WmLdapLoginModule";     
  }
  else
  {
    $self->log("SSL Enabled but not conifigured correctly, reverting to http connector");
    $self->setStartupEnvironment($jetty_port, $secure_port, $jboss_base);
  }
  

 
  
}


# used by update jboss ports to set the environment variables so the templates can be modified accordingly,
# this sets up the env when SSO is enabled using http and https connectors, SharedClientCertificateValve and SharedClientCertificateAuthenticator


sub setStartupEnvironmentSSO
{
  my $self = shift;
  my ($jetty_port, $secure_port, $jboss_base) = @_;

  my $conf_dir = join("/", $jboss_base, "server/default/conf");
  
  my $sso_connector_xml = <<EXML;             
                 <!-- A HTTP/1.1 Connector Configuration For Tomcat 5.5 -->
                 <Connector port="$jetty_port" 
                    address="\${jboss.bind.address}"
                    maxThreads="250" 
                    strategy="ms" 
                    maxHttpHeaderSize="8192"
                    emptySessionPath="true"
                    enableLookups="false" 
                    redirectPort="$secure_port" 
                    acceptCount="100"
                    connectionTimeout="20000" 
                    disableUploadTimeout="true"/>

                  <!-- SSO Connector configuration using the SSL domain keystore  -->
                  <Connector port="$secure_port"  
                      address="\${jboss.bind.address}" 
                      maxThreads="100" 
                      strategy="ms" 
                      maxHttpHeaderSize="8192" 
                      emptySessionPath="true" 
                      scheme="https" 
                      secure="true" 
                      clientAuth="true" 
                      sslProtocol="TLS" 
                      keystoreFile="\${jboss.server.home.dir}/conf/$ENV{SSO_SERVER_KEYSTORE}" 
                      keystorePass="changeit" 
                      truststoreFile="\${jboss.server.home.dir}/conf/$ENV{SSO_SERVER_TRUSTSTORE}" 
                      truststorePass="changeit" 
                      crlFile="\${jboss.server.home.dir}/conf/$ENV{SSO_CRL_FILE}" /> 
EXML

  my $keystore    = join ("/", $ENV{'SSO_STORES_DIR'}, $ENV{'SSO_SERVER_KEYSTORE'});
  my $truststore  = join ("/", $ENV{'SSO_STORES_DIR'}, $ENV{'SSO_SERVER_TRUSTSTORE'});
  my $crl         = join ("/", $ENV{'SSO_STORES_DIR'}, $ENV{'SSO_CRL_FILE'});  
  
  # this is an addtional flag that will remain true of the sso config files are copied over correctly and the env is set up for it
  my $sso_config = 1;
  
  # first we need to copy the SSO config into the conf dir
  unless (copy($keystore,$conf_dir))
  {
    $self->log("WARNING: keystore file not copied from location [$keystore] to [$conf_dir]");
    $sso_config = 0;
  }
  
  unless (copy($truststore,$conf_dir))
  {
    $self->log("WARNING: truststore file not copied from location [$truststore] to [$conf_dir]");
    $sso_config = 0;
  }
  
  unless (copy($crl,$conf_dir))
  {
    $self->log("WARNING: crl file not copied from location [$crl] to [$conf_dir]");
    $sso_config = 0;
  }

  # note that CLIENT_CERT_AUTHENTICATOR, SSO_TOMCAT_VALVE & JAAS_LOGIN_MODULE are set from .profile.oem now so these are not overridden here.
  unless (exists($ENV{'CLIENT_CERT_AUTHENTICATOR'}) && exists($ENV{'SSO_TOMCAT_VALVE'}) && exists($ENV{'JAAS_LOGIN_MODULE'}))
  {
    $self->log("WARNING: SSO Environment variables not correctly set up");
    $sso_config = 0;
  }
  
  if ($sso_config)
  {
    $self->log("SSO Enabled and conifigured correctly, using http/https connectors configured for SSO");
    $ENV{'TOMCAT_CONNECTOR_XML'}= $sso_connector_xml;
    $ENV{'TOMCAT_VALVE_XML'}= '<Valve className="'. $ENV{'SSO_TOMCAT_VALVE'}. '" />';
  }
  else
  {
    $self->log("SSO Enabled but not conifigured correctly, reverting to http connector");
    $self->setStartupEnvironment($jetty_port, $secure_port, $jboss_base);
  }

  # we will use this later to determine what to do with pweb.war
  $self->ssoConfiguredOK($sso_config); 

}


sub log
{
  my $self = shift;
  my ($msg) = @_;
  my $date = `date +[%Y/%m/%d_%H:%M:%S]`;
  chomp($date);
  print "$date $msg \n"; 
}

1; # to allow module compilation